"""These services are part of the Akara regression test suite

They test functionality from the akara.services modules
"""
import os

from amara import tree, parse

import akara
from akara.services import *
from akara.pipeline import *
from akara.registry import register_template, register_services, get_internal_service_url
from akara import request, response

## These are all errors in the simple_service definition.
# They are caught in the server. Do them now because any
# failures means the other services will not be registered
try:
    # Slashes are not (yet) allowed in the path
    @simple_service("GET", "http://example.com/test_args", path="simple_service.with/slashes")
    def spam():
        pass
    raise AssertionError("simple_service.with/slashes was allowed")
except ValueError, err:
    assert "may not contain a '/'" in str(err), err


try:
    # Can only "GET" and "POST" in a simple service
    @simple_service("DELETE", "http://example.com/test_args")
    def spam():
        pass
    raise AssertionError("DELETE was allowed")
except ValueError, err:
    assert "only supports GET and POST methods" in str(err), err

try:
    # Slashes are not (yet) allowed in the path
    @service("http://example.com/test_arg", path="service.with/slashes")
    def spam():
        pass
    raise AssertionError("service.with/slashes was allowed")
except ValueError, err:
    assert "may not contain a '/'" in str(err), err

try:
    # Slashes are not (yet) allowed in the path
    @method_dispatcher("http://example.come/mulimethod", "method_dispatcher.with/slashes")
    def spam():
        pass
    raise AssertionError("method_dispatcher.with/slashes was allowed")
except ValueError, err:
    assert "may not contain a '/'" in str(err), err


## These services are called by the test script

# Make sure the environment is set up
@simple_service("GET", "http://example.com/test")
def test_environment():
    # 
    assert akara.raw_config is not None
    assert akara.module_config("Akara") is not None
    assert akara.module_config("Akara")["Listen"] is not None
    assert (akara.module_config("Akara").get("Listen") ==
            akara.module_config("Akara")["Listen"])
    assert akara.module_config("Akara").get("XYZListen", 123) == 123
    assert akara.module_config("Akara").require("Listen", "SHRDLU")
    try:
        akara.module_config("Akara").require("XYZListen", "SHRDLU")
        raise AssertionError
    except Exception, err:
        assert "SHRDLU" in str(err)
    assert not akara.module_config("XYZAkara")  # should return False
    assert akara.module_config("Akara") # should return True
    

    # simple services can access the WSGI environ this way
    assert request.environ is not None

    # Here's how to override the response fields
    assert response.code.startswith("200 ")
    assert response.headers == []
    return "Good!"

# Make sure the simple_service can specify the path
# (If not given, uses the function's name)
@simple_service("GET", "http://example.com/test", path="test.new.path")
def test_set_path():
    return "Goody!"

# Return a different content-type
@simple_service("GET", "http://example.com/test", content_type="image/gif")
def test_image_gif():
    # A small gif, from http://www.perlmonks.org/?node_id=7974
    # 1x1 pixel red image
    return ('GIF89a\x01\x00\x01\x00\x90\x00\x00\xff\x00\x00\x00'
            '\x00\x00,\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02'
            '\x02\x04\x01\x00;')

# Set the content-type through the response
# Make sure it overrides the one in the simple_service
@simple_service("GET", "http://example.com/test", content_type="image/gif")
def test_dynamic_content_type():
    response.add_header("Content-Type", "chemical/x-daylight-smiles")
    return "c1ccccc1O" # a phenol

# Unicode tests

@simple_service("GET", "http://example.com/test")
def test_unicode_utf8():
    # A simple Unicode string (nothing outside of Latin-1)
    return u"\xc5sa bor i G\xf6teborg"

# Yes, I know I'm not returning text/plain, but I want to make sure
# the simple_service parameter overrides the default, which is based
# on the return being a Unicode string.
@simple_service("GET", "http://example.com/test", "test_unicode_latin1",
                "text/plain", "latin1")
def test_unicode_latin1():
    return u"\xc5sa bor i G\xf6teborg"

# Check that I can override the default content-type specification
@simple_service("GET", "http://example.com/test", None, "text/plain", encoding="utf16")
def test_unicode_utf16():
    response.add_header("Content-Type", "something/strange")
    return u"\xc5sa bor i G\xf6teborg"

# Amara strings
test_document = tree.entity()
node = test_document.xml_append(tree.element(None, "nothing"))
node.xml_attributes["name"] = u"\xc5sa"
node.xml_attributes["city"] = u"G\xf6teborg"
node.xml_append(tree.element(None, "something"))
del node


@simple_service("GET", "http://example.com/test")
def test_xml_utf8():
    return test_document

@simple_service("GET", "http://example.com/test", encoding="latin1")
def test_xml_latin1():
    return test_document

@simple_service("GET", "http://example.com/test", writer="xml-indent")
def test_xml_utf8_indent():
    return test_document

# The default WSGI return types are list of (byte) strings and
# iterator of byte strings. Akara can't do much with these. It assumes
# the results are "text/plain" (if not given) and leave it be.

@simple_service("GET", "http://example.com/test")
def test_list_of_strings():
    return ["This ", "is ", "a ", "test.\n"]

@simple_service("GET", "http://example.com/test")
def test_iterator():
    yield "When shall we three meet again?\n"
    yield "In thunder, lightning, or in rain?\n"
    yield "When the hurlyburly's done,\n"
    yield "When the battle's lost and won.\n"


# Basic args (repeats are not allowed)
@simple_service("GET", "http://example.com/test_args", None, "text/plain")
def test_args(a, b=3):
    return "Hi %s and %s" % (a, b)

# Allow repeated parameters
@simple_service("GET", "http://example.com/test_args", content_type="text/plain",
                allow_repeated_args=True)
def test_repeated_args(a, b=3):
    return "Hello %s and %s" % (a, b)


# Add new headers to the response, including multiple headers with the same name.
@simple_service("GET", "http://example.com/test_args")
def test_add_headers():
    response.add_header("URL", "http://www.xml3k.org/")
    response.add_header("URL", "http://dalkescientific.com/")
    response.add_header("Location", "http://freemix.it/")
    return "Nothing to see here. Move along.\n"

###

@simple_service("GET", "http://example.com/test_echo")
def test_echo_simple_get(**kwargs):
    "This echos a GET request, including the QUERY_STRING"
    for k, v in sorted(kwargs.items()):
        yield "%r -> %r\n" % (k, v)

# This exists only to get two hits in the "/" index
@simple_service("GET", "http://example.com/test_echo")
def test_echo_simple_get2(**kwargs):
    "Hi test_server.py!"
    return test_echo_simple_get(**kwargs)

@simple_service("POST", "http://example.com/test_echo_post")
def test_echo_simple_post(query_body, query_content_type, **kwargs):
    "This echos a POST request, including the query body"
    yield "Content-Type: %r\n" % query_content_type
    yield "Length: %s\n" % len(query_body)
    yield "Body:\n"
    yield repr(query_body) + "\n"


@simple_service("POST", "http://echo_request_headers")
def test_echo_post_headers(query_body, ignore):
    from akara import request
    for k, v in request.environ.items():
        if k.startswith("HTTP_"):
            k = k[5:]
            yield "%s -> %s\n" % (k, v)
    yield "== End of the headers ==\n"


#### '@service' tests

# the 'service' decorator is a thin wrapper over the standard WSGI
# interface Based on code inpection, just about everything is tested
# by the simple_service decorator, so I'm not going to test all the
# error cases.

@service("http://example.com/test_service")
def test_service_no_path(environ, start_response):
    start_response("200 Ok", [("Content-Type", "text/plain")])
    return "this uses the default path\n"

@service("http://example.com/test_service", "test_service_path")
def test_service_with_a_path(environ, start_response):
    start_response("200 Ok", [("Content-Type", "text/fancy")])
    return "this specified a path\n"

#### @method_dispatcher tests

@method_dispatcher("http://example.come/multimethod")
def test_dispatching_with_no_methods():
    "Test method dispatching"


@method_dispatcher("http://example.com/multimethod", path="test_dispatching_get")
def test_dispatching_but_with_a_different_name():
    pass

@test_dispatching_but_with_a_different_name.simple_method("GET")
def say_hi(a="world"):
    return "Hi, " + a + "!"

@method_dispatcher("http://example.com/multimethod")
def test_multimethod():
    "SHRDLU says 'QWERTY'"


try:
    # Some error testing.
    # Although the HTTP spec allows a wider range of HTTP methods,
    # Akara requires that all methods match /[A-Z]+/
    @test_multimethod.method("lowercase", "text/html")
    def do_post(environ, start_response):
        start_response("202 Accepted", [])
        return ""
    raise AssertionError("Why was a lowercase method allowed?")
except ValueError, err:
    assert "HTTP method 'lowercase' value is not valid. It must contain" in str(err), repr(err)

try:
    # Some more error testing
    @test_multimethod.method("A-Z", "text/html")
    def do_post(environ, start_response):
        start_response("202 Accepted", [])
        return ""
    raise AssertionError("Why was a method with a hyphen allowed?")
except ValueError, err:
    assert "HTTP method 'A-Z' value is not valid. It must contain" in str(err), repr(err)

try:
    # simple only works for GET and POST
    @test_multimethod.simple_method("DELETE")
    def do_delete():
        pass
    raise AssertionError("Why can DELETE be a simple_method?")
except ValueError, err:
    assert "only supports GET and POST methods" in str(err), str(err)
    


# NOTE: if this doesn't work (with a 405 error) then the above error tests
# likely failed, raising an AssertionError. Check the error log
@test_multimethod.simple_method("GET", "text/fancy", allow_repeated_args=True)
def do_get(a, b="spam"):
    return "My name is %r. I like %r" % (a, b)

@test_multimethod.simple_method("POST", "text/schmancy")
def do_post(query_body, query_content_type, a="default value"):
    yield "You sent %d bytes of %r and %r\n" % (len(query_body), query_content_type, a)
    yield "First few bytes: %r\n" % (query_body[:20],)

@test_multimethod.method("DELETE")
def do_delete(environ, start_response):
    if environ.get("CONTENT_TYPE") != "chemical/x-daylight-smiles":
        start_response("501 Not Implemented", [("Content-Type", "text/plain")])
        return "give me a SMILES, not a %s" % environ.get("Content-Type")
    s = environ["wsgi.input"].read(6)
    if s != "[U235]":
        start_response("501 Not Implemented", [("Content-Type", "text/plain")])
        return "Where is the kaboom? There was supposed to be an earth-shattering kaboom!"

    start_response("202 Accepted", [("Content-Type", "application/my-xml")])
    return test_document

@test_multimethod.method("TEAPOT")
def do_extension(environ, start_response):
    start_response("418 I'm a teapot", [("Content-Type", "text/plain")])
    return "short and stout"

#####

@method_dispatcher("http://example.com/multimethod")
def test_dispatch_unicode():
    pass

@test_dispatch_unicode.simple_method("GET", encoding="latin1")
def get_latin1():
    return u"\xc5sa bor i G\xf6teborg"

@test_dispatch_unicode.simple_method("POST", encoding="utf-8")
def post_utf8(body, content_type):
    return u"\xc5sa bor i G\xf6teborg"

@method_dispatcher("http://example.com/multimethod")
def test_dispatch_xml():
    pass


@test_dispatch_xml.simple_method("GET", encoding="utf-8", writer="xml")
def get_xml_utf8():
    return test_document

@test_dispatch_xml.simple_method("POST", encoding="utf-16", writer="xml-indent")
def post_xml_indent_utf16(environ, start_response):
    return test_document



### used in test_server.py
_call_count = 0
@simple_service("GET", "http://example.com/count")
def test_get_call_count():
    global _call_count
    print "Currently", _call_count
    s = "%s %s" % (_call_count, os.getpid())
    _call_count += 1
    return s

#### pipelines

@simple_service("POST", "service:rot13")
def test_rot13(query_body, query_content_type):
    return query_body.encode("rot13")

@simple_service("POST", "service:base64-encode")
def test_base64_encode(query_body, query_content_type):
    return query_body.encode("base64")

@simple_service("POST", "service:base64-decode")
def test_base64_decode(query_body, query_content_type):
    return query_body.decode("base64")

import hashlib

@simple_service("POST", "service:md5-hash")
def test_md5(query_body, query_content_type, key=""):
    x = hashlib.md5(key)
    x.update(query_body)
    return x.digest()


register_pipeline("http://dalkescientific.com/hash_encode",
                  "hash_encode",
                  stages = [Stage("service:md5-hash", key="secret"),
                            "service:base64-encode",
                            ])

# This pipeline depends on another pipeline
register_pipeline("http://dalkescientific.com/hash_encode_rot13",
                  "hash_encode_rot13",
                  stages = ["http://dalkescientific.com/hash_encode",
                            "service:rot13",
                            ])

# This pipeline takes a GET as input
@simple_service("GET", "service:get_name")
def test_repeat_get(text="Andrew"):
    return text

@simple_service("POST", "service:count_children")
def test_count_matches(query_body, query_content_type, xslt="*/*"):
    doc = parse(query_body)
    n = len(doc.xml_select(xslt))
    return str(n)

register_pipeline("http://dalkescientific.com/get_hash",
                  "get_hash",
                  stages = ["service:get_name",
                            "service:md5-hash",
                            "service:base64-encode",
                            ])

register_pipeline("http://dalkescientific.com/broken_pipeline",
                  "broken_pipeline1",
                  stages = ["null:missing:stage",
                            "service:rot13",
                            ])

register_pipeline("http://dalkescientific.com/broken_pipeline",
                  "broken_pipeline2",
                  stages = ["http://dalkescientific.com/hash_encode",
                            "null:missing:stage",
                            ])
register_pipeline("http://dalkescientific.com/count_registry",
                  "test_count_registry",
                  stages = ["http://purl.org/xml3k/akara/services/registry",
                            "service:count_children"])

##### Templates

# Define my own template, with the params not in alphabetical order
@simple_service("GET", "urn:akara.test:template-1", path="test.template1",
                query_template = "?name={name}&language={lang?}")
def test_template1(name="Pat", language=None, os="unix"):
    assert language is not None
    assert os == "unix"
    return "%s uses %s on %s" % (name, language, os)

# Get the query_template from parameter inspection
@simple_service("GET", "urn:akara.test:template-2", path="test.template2")
def test_template2(name, language=None, os="unix"):
    assert language is not None
    assert os == "unix"
    return "%s uses %s on %s" % (name, language, os)

# Do not get args - this is a POST
@simple_service("POST", "urn:akara.test:template-3")
def test_template3(name="Ant", language=None, os="unix"):
    assert language is not None
    assert os == "unix"
    return "%s uses %s on %s" % (name, language, os)

# Should not have the "?"
@simple_service("GET", "urn:akara.test:template-4", path="test.template4")
def test_template4():
    return "Here"


@simple_service("GET", "urn:akara.test:template-5")
def test_template5():
    from akara.registry import get_internal_service_url, get_service_url
    import akara.global_config

    yield "server_root: " + akara.global_config.server_root + "\n"
    yield "internal_server_root: " + akara.global_config.internal_server_root + "\n"

    try:
        # Check that the substitution works as expected
    
        for get_url in (get_internal_service_url, get_service_url):
            params = dict(name="Matt", language="C++", os="Linux", lang="kd")
            yield get_url("urn:akara.test:template-1", **params) + "\n"
            yield get_url("urn:akara.test:template-2", **params) + "\n"
            try:
                get_url("urn:akara.test:template-3", **params)
                assert AssertError("Should not get here-3")
            except TypeError:
                pass
            yield get_url("urn:akara.test:template-4", **params) + "\n"

            params = dict(name=u"\xc5sa", language="C&C#", os= u"G\xf6teborg")
            yield get_url("urn:akara.test:template-1", **params) + "\n"
            yield get_url("urn:akara.test:template-2", **params) + "\n"
            yield get_url("urn:akara.test:template-4", **params) + "\n"

            yield get_url("urn:akara.test:template-1", name=u"\xc5sa") + "\n"
            try:
                yield get_url("urn:akara.test:template-1", lang=u"\xc5sa") + "\n"
                raise AssertionError
            except KeyError, err:
                assert "name" in str(err)
    except Exception, err:
        yield "Error!"
        yield repr(err)+"\n"
        yield "spam"
        

##### extra service registration

import akara.global_config

@simple_service("GET", "urn:akara.test:extra_registration")
def extra_registration():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<services>
  <service ident="urn:akara.test:extra_echo">
     <path template="%stest_echo_simple_get?foo={bar}">test_echo_simple_get</path>
  </service>
</services>
""" % (akara.global_config.internal_server_root,)


register_template("urn:service_reg:1",
                  akara.global_config.internal_server_root +
                  "test.template1?name={x}&language=FORTRAN")

# Attempt to override a local handler.
# (Won't be able to. Local handlers always have priority over external ones.)
register_template("http://purl.org/xml3k/akara/services/registry",
                  "http://qerwerw.eryqp:0/")

# GRR! This doesn't work. I can't register myself during import
# time because the server isn't running during import.
def _delayed_install():
    register_services(akara.global_config.internal_server_root + "extra_registration")

@simple_service("GET", "urn.akara.test:extra_call")
def test_extra_call(service_id, **kwargs):
    try:
        # Hack solution so I can call myself.
        _delayed_install()
        
        import urllib
        url = get_internal_service_url(service_id, **kwargs)
        f = urllib.urlopen(url)
        body = f.read()
        yield "URL: " + url + "\n"
        yield body
    except Exception, err:
        yield str(err)
        raise



########################################################################
# akara/resource/repository/__init__.py
"""

"""
import sqlite3
from cStringIO import StringIO


#__all__ = ['driver']

class driver(object):
    """
    Typical usage:
    
    driver = akara.resource.repository.driver(sqlite3.connect(conn_str))
    """
    def __init__(self, conn):
        self._conn = conn
        pass

    def create_resource(self, content, metadata):
        """
        content - the actual content of the resource
        metadata - a dictionary

        We will generate the id.
        If e.g. they use hierarchical path aliases, that would be part of the metadata
        """
        c = self._conn.cursor()
        #http://www.sqlite.org/faq.html#q1
        c.execute('INSERT into resources values(NULL, ?)', (content,))
        new_id = c.lastrowid
        #c.execute('SELECT max(id) FROM resources')
        #rowid = c.next()[0]
        #print 'past inserted', rowid
        #Use executemany?
        for key, value in metadata.iteritems():
            c.execute('insert into metadata values(?, ?, ?)', (new_id, key, value))
        self._conn.commit()
        return new_id

    def has_resource(self, id):
        """
        id = ID of the resource to check
        
        Return a boolean
        """
        c = self._conn.cursor()
        c.execute('select content from resources where id=?', (id,))
        try:
            c.next()
            resource_exists = True
        except:
            resource_exists = False
        c.close()
        return resource_exists

    def get_resource(self, id):
        """
        id = ID of the resource to get
        
        return a stream and an iterator over the metadata dict
        """
        c = self._conn.cursor()
        c.execute('select content from resources where id=?', (id,))
        data = c.fetchone()
        if not data:
            c.close()
            return None, None
        data = data[0]
        c.execute('select key, value from metadata where id=?', (id,))
        metadata = dict(c)
        #for row in c:
        c.close()
        #stream = StringIO(data)
        return data, metadata

    def update_resource(self, id, content=None, metadata=None):
        """
        id - ID of the resource to update
        content - text or stream with new resource content, or None to leave content untouched
        metadata - dict of metadata to be added/updated
        
        return a stream and an iterator over the metadata dict
        """
        c = self._conn.cursor()
        if content is not None:
            c.execute('update resources set content=? where id=?', (content, id))
        #Use executemany?
        for key, value in metadata.iteritems():
            c.execute('update metadata set key=?, set value=? where id=?', (key, value, id,))
        self._conn.commit()
        return

    def delete_resource(self, id):
        """
        id = ID of the resource to delete
        """
        c = self._conn.cursor()
        c.execute('DELETE from resources where id=?', (id,))
        c.execute('DELETE from metadata where id=?', (id,))
        self._conn.commit()
        return

    def get_metadata(self, id):
        """
        id = ID of the resource to get
        
        return a stream and an iterator over the metadata dict
        """
        c = self._conn.cursor()
        #c.execute('select content from resources where id=?', (id,))
        #data = c.next()[0]
        c.execute('select key, value from metadata where id=?', (path,))
        metadata = dict(c)
        #for row in c:
        c.close()
        return metadata

    def lookup(self, key, value):
        """
        key - metadata key
        value - metadata value to match
        """
        c = self._conn.cursor()
        #c.execute('select content from resources where id=?', (id,))
        #data = c.next()[0]
        c.execute('select id from metadata where key=? and value=?', (key, value))
        result = [ r[0] for r in c ]
        #metadata = dict(c)
        #for row in c:
        c.close()
        return result

    def orderby(self, key, count=None, reverse=False):
        """
        id = ID of the resource to get
        
        return a stream and an iterator over the metadata dict
        """
        direction = 'DESC' if reverse else 'ASC'
        c = self._conn.cursor()
        #c.execute('select content from resources where id=?', (id,))
        #data = c.next()[0]
        if count:
            c.execute('select id, value from metadata where key=? order by value %s limit ?'%(direction), (key, count))
        else:
            c.execute('select id, value from metadata where key=? order by value %s'%(direction), (key,))
        result = [ r[0] for r in c ]
        #metadata = dict(c)
        #for row in c:
        c.close()
        return result

    #def create_alias(self, alias, id):
    #    """
    #    alias - the alias for the resource
    #    id - the resource's ID
    #    """
    #    c = self._conn.cursor()
    #    c.execute('INSERT into alias values(?, ?)', (alias, id))
    #    self._conn.commit()
    #    #def lookup_alias(self, alias):
    #    #c.execute('SELECT * from alias values(?, ?)', (alias, id))
    #    return new_id

    @staticmethod
    def init_db(conn):
        c = conn.cursor()

        # Create table
        c.execute('''create table resources
        (id INTEGER PRIMARY KEY, content TEXT)''')
        c.execute('''create table metadata
        (id INTEGER, key TEXT, value TEXT)''')
        #c.execute('''create table aliases
        #(alias TEXT, id INTEGER)''')
        
        c.close()
        pass



from akara.opensearch import apply_template

import unittest

class Tests(unittest.TestCase):
    def raisesTypeError(self, f, *args, **kwargs):
        try:
            raise AssertionError( f(*args, **kwargs) )
        except TypeError, err:
            return str(err)

    #### These test the public API
    def test_no_expansion(self):
        self.assertEquals(apply_template("http://example.com/osd.xml"),
                          "http://example.com/osd.xml")
        self.assertEquals(apply_template(u"http://example.com/osd.xml"),
                          "http://example.com/osd.xml")
        
    def test_single_term(self):
        self.assertEquals(apply_template("http://example.com/search?q={searchTerms}",
                                         searchTerms="Andrew"),
                          "http://example.com/search?q=Andrew")
    def test_spaces(self):
        self.assertEquals(apply_template("http://example.com/search?q={searchTerms}",
                                         searchTerms="Andrew Dalke"),
                          "http://example.com/search?q=Andrew+Dalke")

    def test_many(self):
        T = "{scheme}://{host}:{port}/{path}?q={arg}#{hash}"
        self.assertEquals(apply_template(T, scheme="gopher",
                                         host="hole", port="70",
                                         path="somewhere/else",
                                         arg="spam & eggs",
                                         hash="browns"),
                          "gopher://hole:70/somewhere%2Felse?q=spam+%26+eggs#browns")
    def test_failure(self):
        self.assertRaises(KeyError, apply_template, "http://{host}/")

    def test_optional(self):
        self.assertEquals(apply_template("http://example.com/{arg?}"),
                          "http://example.com/")
        self.assertEquals(apply_template("http://example.com/{arg?}{arg?}"),
                          "http://example.com/")
        self.assertEquals(apply_template("http://example.com/{arg?}{arg?}", arg="X"),
                          "http://example.com/XX")

    def test_optional_in_scheme(self):
        err = self.raisesTypeError(apply_template, "{scheme?}://example.com/",
                                   scheme="gopher")
        assert err.startswith("URI scheme cannot be an optional template variable")

    def test_optional_in_hostname(self):
        err = self.raisesTypeError(apply_template, "scheme://{hostname?}.com/",
                                   hostname="example")
        assert err.startswith("URI hostname cannot contain an optional template variable")


    def test_unicode_failures(self):
        # Scheme cannot include Unicode
        self.assertRaises(UnicodeEncodeError, apply_template,
                          "{scheme}://blah", scheme=u"Espa\u00F1a")
        # Don't allow non-ascii characters except in the domain
        self.assertRaises(UnicodeEncodeError, apply_template,
                          u"http://google.com/search?q=Espa\u00F1a")
        self.assertRaises(UnicodeEncodeError, apply_template,
                          u"http://\u00F1{q}@example.com")
        

    def test_multiple_args_in_scheme(self):
        for (target, expected_err) in (
            ("{scheme}{scheme}://example.com/", "text or a single template field"),
            ("blah{scheme}://example.com", "text or a single template field"),
            ("blah/", "Missing or unparsable"),
            ):
            err = self.raisesTypeError(apply_template, target, scheme="http")
            assert expected_err in err, (target, expected_err, err)

    def test_unicode_in_host(self):
        # OpenSearch template does not allow this. I do.
        self.assertEquals(apply_template("https://{host}.name/", host=u"Espa\u00F1a"),
                          "https://xn--espaa-rta.name/")
        # I also allow Unicode in the host name (but only there!)
        self.assertEquals(apply_template(u"https://{host}.name/", host=u"Espa\u00F1a"),
                          "https://xn--espaa-rta.name/")
        self.assertEquals(apply_template(u"https://{host}.a\u00F1o/",
                                         host=u"Espa\u00F1a"),
                          "https://xn--espaa-rta.xn--ao-zja/")

    def test_unicode_in_host_part(self):
        # I double checked the encoding with Safari. Seems to be right
        self.assertEquals(apply_template(u"https://{host}.Espa\u00F1a/", host=u"a\u00F1o"),
                          "https://xn--ao-zja.xn--espaa-rta/")
        self.assertEquals(apply_template(u"http://{host}.\u00F1.name/", host=u"Espa\u00F1a"),
                          "http://xn--espaa-rta.xn--ida.name/")

    def test_unicode_in_path(self):
        # I double checked the encoding with Safari. Seems to be right
        self.assertEquals(apply_template("http://google.com/search?q={q}",
                                         q=u"Espa\u00F1a"),
                          "http://google.com/search?q=Espa%C3%B1a")
        self.assertEquals(apply_template(u"http://google.com/search?q={q}",
                                         q=u"Espa\u00F1a"),
                          "http://google.com/search?q=Espa%C3%B1a")


    def test_unicode_in_username(self):
        self.assertEquals(apply_template("http://{q}@example.com",
                                         q=u"Espa\u00F1a"),
                          "http://Espa%C3%B1a@example.com")


    def test_using_prefix(self):
        for uri in ("{spam:x}://dalke:blah@example.com/something",
                    "HTTP://{spam:x}:blah@example.com/something",
                    "HTTP://{spam:x}:blah@example.com/something",
                    "HTTP://dalke:{spam:x}@example.com/something",
                    "HTTP://dalke:blah@{spam:x}.com/something",
                    "HTTP://dalke:blah@example.com/{spam:x}"):
            err = self.raisesTypeError(apply_template, uri)
            assert err.startswith("Template prefix not supported in Akara (in '"), err
            # While this should work just fine
            t = apply_template(uri.replace("spam:", ""), x="123")

    def test_missing_slashes(self):
        for uri in ("http:spam",
                    "http:/spam",
                    "http:/?spam"):
            err = self.raisesTypeError(apply_template, uri)
            assert err.startswith("Missing required '//' in URI"), err
    def test_missing_hostname(self):
        for uri in ("http:///spam/",
                    "http:///spam",
                    "http://dalke@/spam",
                    "http://:@/spam",
                    "http://?spam"):
            err = self.raisesTypeError(apply_template, uri)
            assert err.startswith("Akara requires a hostname in the template"), err

    def test_empty_template_name(self):
        for uri in ("{}://example.com/",
                    "http://{}/",
                    "http://example.com?{}"):
            err = self.raisesTypeError(apply_template, uri)
            assert err.startswith("Empty template variable in '")
                    
    def test_bad_templates(self):
        for uri, c in (("{scheme://example.com/whatever}", "{"),
                       ("{scheme}://example.com/whatever}", "}"),
                       ("http://example.com?whatever=asdf#}", "}"),
                       ("http://example.com?whatever=asdf#{", "{")):
            err = self.raisesTypeError(apply_template, uri)
            assert err.startswith("Unexpected '" + c + "' found in URI"), err

    def test_port_errors(self):
        err = self.raisesTypeError(apply_template, "http://localhost:/")
        assert err.startswith("Port must be either a number or a template name")
        err = self.raisesTypeError(apply_template, "http://localhost:/a")
        assert err.startswith("Port must be either a number or a template name")
        err = self.raisesTypeError(apply_template, "http://localhost:{port}a/")
        assert err.startswith("Port may not contain anything after the template name"), err
        err = self.raisesTypeError(apply_template, "http://localhost:{port}/", port="q")
        assert err.startswith("Port template parameter 'port' is not an integer ('q')"), err
        err = self.raisesTypeError(apply_template, "http://localhost:{port}/", port="-80")
        assert err.startswith("Port template parameter 'port' is not an integer ('-80')")

    def test_port(self):
        self.assertEquals(apply_template("http://localhost:8765/"),
                          "http://localhost:8765/")
        self.assertEquals(apply_template("http://localhost:{port}/", port=8080),
                          "http://localhost:8080/")
        self.assertEquals(apply_template("http://localhost:{port}/", port="8080"),
                          "http://localhost:8080/")
        self.assertEquals(apply_template("http://localhost:{port}/abc", port=""),
                          "http://localhost/abc")
        self.assertEquals(apply_template("http://localhost:{port?}/?q"),
                          "http://localhost/?q")
        self.assertEquals(apply_template("http://localhost:{port?}/?q", port="123"),
                          "http://localhost:123/?q")

if __name__ == "__main__":
    unittest.main()



#akara.resource.index
"""


"""
#self.policy - instance of L{akara.policy.manager}

__all__ = ['simple_xpath_index']

import amara
from amara.tree import node
from amara.lib.util import *
from amara.namespaces import AKARA_NAMESPACE

def simple_xpath_index(self, content, patterns):
    results = {}
    doc = content if isinstance(content, node) else amara.parse(source)
    for pname, xpath in patterns.iteritems():
        result = doc.xml_select(pattern)
        results[pname] = simplify(result)
    return results





"""Deal with the command-line interface

This is an internal module and should not be called from other libraries.

"""

import sys
import os
import signal
import shutil

from akara.thirdparty import argparse
from akara import read_config, run

def get_pid(args):
    try:
        settings, config = read_config.read_config(args.config_filename)
    except read_config.Error, err:
        raise SystemExit(str(err))

    pid_file = settings["pid_file"]
    try:
        f = open(pid_file)
    except IOError, err:
        raise SystemExit("Could not open Akara PID file: %s" % (err,))
    pid = f.readline()
    if not pid:
        raise SystemExit("Empty Akara PID file: %r" % (pid_file,))
    try:
        return int(pid) 
    except ValueError:
        raise SystemExit("Akara PID file %r does not contain a PID (%r)" %
                         (pid_file, pid))

def start(args):
    run.main(args)

def stop(args):
    pid = get_pid(args)
    os.kill(pid, signal.SIGTERM)

def restart(args):
    pid = get_pid(args)
    os.kill(pid, signal.SIGHUP)

def status(args):
    config_filename = read_config.DEFAULT_SERVER_CONFIG_FILE
    if args.config_filename is not None:
        config_filename = args.config_filename

    print "  == Akara status =="
    print "Configuration file:", repr(config_filename)
    try:
        settings, config = read_config.read_config(config_filename)
    except read_config.Error, err:
        print "** ERROR **:", str(err)
        raise SystemExit(1)

    print "Error log file:", repr(settings["error_log"])
    print "Access log file:", repr(settings["access_log"])

    pid_file = settings["pid_file"]
    print "PID file:", repr(pid_file)
    try:
        line = open(pid_file).readline()
    except IOError, err:
        # It's fine if the PID file isn't there
        if not os.path.exists(pid_file):
            print "PID file does not exist"
            print "Akara is not running"
        else:
            print "*** Cannot open PID file:", err
            raise SystemExit(1)
    else:
        try:
            pid = int(line)
        except ValueError, err:
            print "***  Unable to parse the PID from the PID file:", err
            raise SystemExit(1)
        try:
            os.kill(pid, 0)
        except OSError:
            print "PID is", pid, "but no process found with that PID"
            print "Akara is not running"
        else:
            print "PID is", pid, "and there is a process with that PID"
            # XXX try to connect to the server?
            print "Akara is running"


def setup_config_file():
    _setup_config_file(read_config.DEFAULT_SERVER_CONFIG_FILE)

# This function is called by test code.
# It is not part of the external API.
def _setup_config_file(default_file):
    if os.path.exists(default_file):
        print "Configuration file already exists at", repr(default_file)
    else:
        print "Copying reference configuration file to", repr(default_file)
        dirname = os.path.dirname(default_file)
        if not os.path.exists(dirname):
            print "  Creating directory", dirname
            try:
                os.makedirs(dirname)
            except OSError, err:
                raise SystemExit("Cannot make directory: %s" % err)

        # Using 'read_config.__file__' because it was handy
        akara_config = os.path.join(os.path.dirname(read_config.__file__),
                                    "akara.conf")
        try:
            shutil.copy(akara_config, default_file)
        except IOError, err:
            raise SystemExit("Cannot copy file: %s" % err)

def setup_directory_for(what, dirname):
    if os.path.isdir(dirname):
        if what[0] > "Z":
            what = what[0].upper() + what[1:]
        print "%s directory exists: %r" % (what, dirname)
    else:
        try:
            os.makedirs(dirname)
        except OSError, err:
            raise SystemExit("Cannot make %s directory: %s" % (what, err))
        print "Created %s directory: %r" % (what, dirname)

def setup(args):
    if not args.config_filename:
        setup_config_file()
    settings, config = read_config.read_config(args.config_filename)

    dirname = os.path.dirname
    setup_directory_for("error log", dirname(settings["error_log"]))
    setup_directory_for("access log", dirname(settings["access_log"]))
    setup_directory_for("PID file", dirname(settings["pid_file"]))
    setup_directory_for("extension modules", settings["module_dir"])

    print
    print "Akara environment set up. To start Akara use:"
    print "    akara start"


# This function is not multi-process safe. It's meant to be
# called by hand during the development process
def error_log_rotate(args):
    import datetime
    settings, config = read_config.read_config(args.config_filename)
    error_log = settings["error_log"]

    ext  = ""
    i = 0
    timestamp = datetime.datetime.now().isoformat().split(".")[0]
    template = error_log + "." + timestamp
    archived_error_log = template
    while os.path.exists(archived_error_log):
        i += 1
        archived_error_log = template + "_" + str(i)

    try:
        os.rename(error_log, archived_error_log)
    except OSError, err:
        if not os.path.exists(error_log):
            print "No error log found at %r" % error_log
        else:
            raise
    else:
        print "Rotated log file from %r to %r" % (error_log, archived_error_log)
    

######################################################################

# Handle the command-line arguments

parser = argparse.ArgumentParser(prog="akara", add_help=False)

parser.add_argument("-f", "--config-file", metavar="FILE", dest="config_filename",
                    help="read configuration data from FILE")


# I didn't like how argparse put the --help and --version options first.
# I didn't like how it uses -v as a variation of --version.
# So, do it myself.
parser.add_argument("-h", "--help", action="help", default=argparse.SUPPRESS,
                    help=argparse._("show this help message and exit"))

parser.version = "akaractl version 2.0"
parser.add_argument("--version", action="version", default=argparse.SUPPRESS,
                    help=argparse._("show program's version number and exit"))

#### Commands for start, stop, etc.

subparsers = parser.add_subparsers(title="The available server commands are")

parser_start = subparsers.add_parser("start", help="start Akara (use -X for debug mode)")
parser_start.add_argument("-X", dest="debug", action="store_true",
                          help="start in debug mode")
parser_start.add_argument("-f", dest="skip_pid_check", action="store_true",
                          help="do not check for an existing PID file")
parser_start.set_defaults(func=start)

parser_stop = subparsers.add_parser("stop", help="stop an Akara server")
parser_stop.set_defaults(func=stop)

parser_restart = subparsers.add_parser("restart", help="restart an Akara server")
parser_restart.set_defaults(func=restart)

parser_status = subparsers.add_parser("status", help="display a status report")
parser_status.set_defaults(func=status)

parser_setup = subparsers.add_parser("setup", help="set up directories and files for Akara")
parser_setup.set_defaults(func=setup)

# There may be an "akara log rotate" in the future, along perhaps with
# "akara log tail", "akara log filename" and other options. There's
# not yet enough call for those and the following doesn't interfere
# with the possibility (excepting non-orthagonality).

parser_setup = subparsers.add_parser("rotate",
                                     help="rotate out the current Akara error log")
parser_setup.set_defaults(func=error_log_rotate)


def main(argv):
    args = parser.parse_args(argv[1:])
    args.func(args)



"""akara.services - adapters to use Python functions as WSGI handlers

This module is meant to be used by functions in an Akara extension
module.

"""

import httplib
import warnings
import functools
import cgi
import inspect
from cStringIO import StringIO
from xml.sax.saxutils import escape as xml_escape

from BaseHTTPServer import BaseHTTPRequestHandler
http_responses = BaseHTTPRequestHandler.responses
del BaseHTTPRequestHandler

from amara import tree, writers

from akara import logger, registry

__all__ = ("service", "simple_service", "method_dispatcher")

ERROR_DOCUMENT_TEMPLATE = """<?xml version="1.0" encoding="ISO-8859-1"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
<head>
  <title>%(reason)s</title>
</head>
<body>
  <h1>%(reason)s</h1>
  <p>
  %(message)s
  </p>
  <h2>Error %(code)s</h2>
</body>
</html>
"""

class _HTTPError(Exception):
    "Internal class."
    # I don't think the API is quite right.
    # Should code be a number or include the reason in a string, like "200 OK"?
    def __init__(self, code, message=None):
        assert isinstance(code, int) # Being a bit paranoid about the API
        self.code = code
        self.reason, self.message = http_responses[code]
        if message is not None:
            self.message = message
        self.text = ERROR_DOCUMENT_TEMPLATE % dict(code=self.code,
                                                   reason=xml_escape(self.reason),
                                                   message=xml_escape(self.message))
        self.headers = [("Content-Type", "text/html")]
    def make_wsgi_response(self, environ, start_response):
        start_response("%s %s" % (self.code, self.reason), self.headers)
        return [self.text]

class _HTTP405(_HTTPError):
    def __init__(self, methods):
        _HTTPError.__init__(self, 405)
        self.headers.append( ("Allow", ", ".join(methods)) )


# Pull out any query arguments and set up input from any POST request
def _get_function_args(environ, allow_repeated_args):
    request_method = environ.get("REQUEST_METHOD")
    if request_method == "POST":
        try:
            request_length = int(environ["CONTENT_LENGTH"])
        except (KeyError, ValueError):
            raise _HTTPError(httplib.LENGTH_REQUIRED)
        if request_length < 0:
            raise _HTTPError(httplib.BAD_REQUEST)
        request_bytes = environ["wsgi.input"].read(request_length)
        request_content_type = environ.get("CONTENT_TYPE", None)
        args = (request_bytes, request_content_type)
    else:
        args = ()

    # Build up the keyword parameters from the query string
    query_string = environ["QUERY_STRING"]
    kwargs = {}
    if query_string:
        qs_dict = cgi.parse_qs(query_string)
        if allow_repeated_args:
            kwargs = qs_dict
        else:
            for k, v in qs_dict.iteritems():
                if len(v) == 1:
                    kwargs[k] = v[0]
                else:
                    raise _HTTPError(400, 
   message="Using the %r query parameter multiple times is not supported" % (k,))
            
    return args, kwargs

######

def new_request(environ):
    "prepare the akara.request and akara.response environment for a new request"
    from akara import request, response
    request.environ = environ
    response.code = "200 OK"
    response.headers = []

def send_headers(start_response, default_content_type, content_length):
    "Send the WSGI headers, using values from akara.request.*"
    from akara import response
    code = response.code
    if isinstance(code, int):
        reason = http_responses[code][0]
        code = "%d %s" % (code, reason)
    has_content_type = False
    has_content_length = False
    for k, v in response.headers:
        k = k.lower()
        if k == "content-type":
            has_content_type = True
        elif k == "content-length":
            has_content_length = True

    if not has_content_type:
        response.headers.append( ("Content-Type", default_content_type) )
    if not has_content_length and content_length is not None:
        response.headers.append( ("Content-Length", content_length) )

    start_response(code, response.headers)


def convert_body(body, content_type, encoding, writer):
    if isinstance(body, str):
        if content_type is None:
            content_type = "text/plain"
        return [body], content_type, len(body)

    if isinstance(body, tree.entity):
        # XXX have Amara tell me the content type (with encoding)
        # This is trac #29
        if content_type is None:
            if "html" in writer.lower():
                content_type = "text/html"
            else:
                content_type = "application/xml"
        w = writers.lookup(writer)
        body = body.xml_encode(w, encoding)
        return [body], content_type, len(body)

    if isinstance(body, unicode):
        body = body.encode(encoding)
        if content_type is None:
            content_type = "text/plain; charset=%s" % (encoding,)
        return [body], content_type, len(body)

    # Probably one of the normal WSGI responses
    if content_type is None:
        content_type = "text/plain"
    return body, content_type, None


# The HTTP spec says a method can be and 1*CHAR, where CHAR is a
# US-ASCII character excepting control characters and "punctuation".
# (like '(){}' and even ' '). We're a bit more strict than that
# because we haven't seen people use words like "get".
def _check_is_valid_method(method):
    min_c = min(method)
    max_c = max(method)
    if min_c < 'A' or max_c > 'Z':
        raise ValueError("HTTP method %r value is not valid. "
                         "It must contain only uppercase ASCII letters" % (method,))

def _no_slashes(path):
    if path is not None and "/" in path:
        # Really these are more like mount points
        raise ValueError("service paths may not contain a '/'")

def ignore_start_response(status, response_headers, exc_info=None):
    pass

def _make_query_template(func):
    argspec = inspect.getargspec(func)
    if argspec.varargs is not None or argspec.keywords is not None:
        # Can't handle *args or **kwargs in the parameter list
        return None
    if not argspec.args:
        return ""
    num_required = len(argspec.args) - len(argspec.defaults or ())
    arg_info = [(arg, i >= num_required) for (i, arg) in enumerate(argspec.args)]
    # I present these in alphabetical order to reduce template changes
    # should the parameter list change.
    arg_info.sort()
    terms = []
    for arg, is_optional in arg_info:
        if is_optional:
            fmt = "%s={%s?}"
        else:
            fmt = "%s={%s}"
        terms.append( fmt % (arg, arg) )
    return "?" + "&".join(terms)


def _handle_notify(environ, f, service_list):
    for service_id in service_list:
        service = registry.get_a_service_by_id(service_id)
        service_environ = environ.copy()
        service_environ["PATH_INFO"] = service.path
        f.seek(0)
        new_request(service_environ)
        try:
            service.handler(service_environ, ignore_start_response)
        except Exception:
            raise
            # XXX
            pass

FROM_ENVIRON = object()
def _handle_notify_before(environ, body, service_list):
    if not service_list:
        return
    if body is FROM_ENVIRON:
        body = environ["wsgi.input"].read()
    f = StringIO(body)
    environ["wsgi.input"] = f
    _handle_notify(environ, f, service_list)
    f.seek(0)

def _handle_notify_after(environ, result, service_list):
    if not service_list:
        return result
    f = StringIO()
    for block in result:
        f.write(block)
    # XXX ALso need to set the CONTENT_TYPE (and others?)
    environ["CONTENT_LENGTH"] = f.tell()
    environ["wsgi.input"] = f
    _handle_notify(environ, f, service_list)
    f.seek(0)
    return f

###### public decorators

## Guide to help in understanding
# @service(*args) -> returns a service_wrapper
#
# @service(*args)  
# def func(): pass  -> returns a wrapper() which calls func


def service(service_id, path=None,
            content_type=None,encoding="utf-8", writer="xml",
            pipelines = None,
            query_template = None,
            wsgi_wrapper=None,
            notify_before = None,
            notify_after = None):
    _no_slashes(path)
    def service_wrapper(func):
        @functools.wraps(func)
        def wrapper(environ, start_response):
            _handle_notify_before(environ, FROM_ENVIRON, notify_before)
            # 'service' passes the WSGI request straight through
            # to the handler so there's almost no point in
            # setting up the environment. However, I can conceive
            # of tools which might access 'environ' directly, and
            # I want to be consistent with the simple* interfaces.
            new_request(environ)
            result = func(environ, start_response)

            # You need to make sure you sent the correct content-type!
            result, ctype, length = convert_body(result, None, encoding, writer)
            result = _handle_notify_after(environ, result, notify_after)
            return result

        pth = path
        if pth is None:
            pth = func.__name__

        # If an outer WSGI wrapper was specified, place it around the service wrapper being created
        if wsgi_wrapper:
            wrapper = wsgi_wrapper(wrapper)

        #For purposes of inspection (not a good idea to change these otherwise you'll lose sync with the values closed over)
        wrapper.content_type = content_type
        wrapper.encoding = encoding
        wrapper.writer = writer

        registry.register_service(service_id, pth, wrapper, query_template=query_template)
        return wrapper
    return service_wrapper


## Guide to help in understanding
# @simple_service(*args) -> returns a service_wrapper
#
# @simple_service(*args)  
# def func(): pass  -> returns a wrapper() which calls func

def simple_service(method, service_id, path=None,
                   content_type=None, encoding="utf-8", writer="xml",
                   allow_repeated_args=False,
                   query_template=None,
                   wsgi_wrapper=None,
                   notify_before=None, notify_after=None):
    """Add the function as an Akara resource

    These affect how the resource is registered in Akara
      method - the supported HTTP method (either "GET" or "POST")
      service_id - a string which identifies this service; should be a URL
      path - the local URL path to the resource (must not at present
           contain a '/') If None, use the function's name as the path.
      query_template - An Akara URL service template (based on OpenSource; see akara.opensource)
           Can be used to help consumers compose resources withing this service.  The same
           template is used for all HTTP methods

    These control how to turn the return value into an HTTP response
      content_type - the response content-type. If not specified, and if
          "Content-Type" is not listed in akara.response.headers then infer
          the content-type based on what the decorated function returns.
          (See akara.services.convert_body for details)
      encoding - Used to convert a returned Unicode string or an Amara tree
          to the bytes used in the HTTP response
      writer - Used to serialize the Amara tree for the HTTP response.
          This must be a name which can be used as an Amara.writer.lookup.

    This affects how to convert the QUERY_STRING into function call parameters
      allow_repeated_args - The query string may have multiple items with the
          same name, as in "?a=x&a=y&a=z&b=w". If True, this is converted into
          a function call parameter like "f(a=['x','y','z'], b=['w'])". If
          False then this is treated as an error. Suppose the query string
          contains no repeated arguments, as in "?a=x&b=w". If
          allow_repeated_args is True then the function is called as
          as "f(a=['x'], b=['w'])" and if False, like "f(a='x', b='w')".
    
    A simple_service decorated function can get request information from
    akara.request and use akara.response to set the HTTP reponse code
    and the HTTP response headers.

    Here is an example of use:

      @simple_service("GET", "http://example.com/get_date")
      def date(format="%Y-%m-%d %H:%M:%S"):
          '''get the current date'''
          import datetime
          return datetime.datetime.now().strftime(format)

    which can be called with URLs like:

      http://localhost:8880/date
      http://localhost:8880/date?format=%25m-%25d-%25Y

    Integration with other WSGI components:

    The @simple_service decorator creates and returns a low-level handler 
    function that conforms to the WSGI calling conventions. However,
    it is not safe to directly use the resulting handler with arbitrary
    third-party WSGI components (e.g., to wrap the Akara handler with
    an WSGI middleware component).   This is because Akara handlers return
    values other than sequences of byte-strings.  For example, they might
    return XML trees, Unicode, or other data types that would not be 
    correctly interpreted by other WSGI components.

    To integrate other WSGI components with Akara, use the wsgi_wrapper
    argument to @simple_service.  For example:

    def wrapper(app):
        # Create an WSGI wrapper around WSGI application app
        ...
        return wrapped_app

    @simple_service("GET", "http://example.com/get_date", wsgi_wrapper=wrapper)
    def date(format):
        ...
    When specified, Akara will do the following:

         - Arrange to have the wsgi_wrapper placed at the outermost layer 
           of Akara's processing.   That is, control will pass into 
           the WSGI wrapper before any Akara-specific processing related
           to the @simple_service handler takes place.

         - Ensure that all output returned back to the WSGI wrapper
           strictly conforms to the WSGI standard (is a sequence of bytes)

    The wrapper function given with wsgi_wrapper should accept a function
    as input and return an WSGI application as output.  This application
    should be a callable that accepts (environ, start_response).

    See implementation notes in the code below.

"""
    _no_slashes(path)
    _check_is_valid_method(method)
    if method not in ("GET", "POST"):
        raise ValueError(
            "simple_service only supports GET and POST methods, not %s" % (method,))

    def service_wrapper(func):
        @functools.wraps(func)
        def wrapper(environ, start_response):
            try:
                if environ.get("REQUEST_METHOD") != method:
                    if method == "GET":
                        raise _HTTP405(["GET"])
                    else:
                        raise _HTTP405(["POST"])
                args, kwargs = _get_function_args(environ, allow_repeated_args)
            except _HTTPError, err:
                return err.make_wsgi_response(environ, start_response)
            if args:
                body = args[0]
            else:
                body = ""
            _handle_notify_before(environ, body, notify_before)

            new_request(environ)
            result = func(*args, **kwargs)

            result, ctype, clength = convert_body(result, content_type, encoding, writer)
            send_headers(start_response, ctype, clength)
            result = _handle_notify_after(environ, result, notify_after)
            return result

        pth = path
        if pth is None:
            pth = func.__name__

        # Construct the default query template, if needed and possible.
        qt = query_template
        if qt is None and method == "GET" and not allow_repeated_args:
            qt = _make_query_template(func)
        if qt is not None:
            qt = pth + qt

        # If an wsgi_wrapper was given, wrapper the service wrapper with it
        if wsgi_wrapper:
           wrapper = wsgi_wrapper(wrapper)

        #For purposes of inspection (not a good idea to change these otherwise you'll lose sync with the values closed over)
        wrapper.content_type = content_type
        wrapper.encoding = encoding
        wrapper.writer = writer

        registry.register_service(service_id, pth, wrapper, query_template=qt)
        return wrapper
    return service_wrapper

# XXX idea for the majority of services which deal with XML
# @xml_service("http://example.com/cool_xml", "cool")
# def cool(xml_tree, param1):
#   ...
#   return xml_tree

#def xml_service()


## Use for services which dispatch based on HTTP method type (GET, POST, ...)

# Nomenclature: the service is identified by its service id.
# All handlers for a given service id implement a given protocol.
# Use a method_dispatcher when a service does different things
# based on the HTTP method (GET, POST, ...) and you want a
# different Python function to handle each method.

class service_method_dispatcher(object):
    """WSGI dispatcher based on request HTTP method

    This is an internal class. You should not need to use it.
    """
    def __init__(self, path, wsgi_wrapper=None):
        self.path = path
        self.method_table = {}
        self.wsgi_wrapper = wsgi_wrapper
    def add_handler(self, method, handler):
        if method in self.method_table:
            logger.warn("Replacing %r method handler for %r"  %
                        (method, self.path))
        else:
            logger.info("Created %r method handler for %r" %
                        (method, self.path))
        # If an outer WSGI wrapper was specified, wrap it around the handler method
        if self.wsgi_wrapper:
            handler = self.wsgi_wrapper(handler)

        self.method_table[method] = handler
    def __call__(self, environ, start_response):
        method = environ.get("REQUEST_METHOD")
        handler = self.method_table.get(method, None)
        if handler is not None:
            return handler(environ, start_response)
        err = _HTTP405(sorted(self.method_table.keys()))
        return err.make_wsgi_response(environ, start_response)
    
    def head_method(self, environ, start_response):
        handler = self.method_table.get("GET",None)
        if handler is not None:
            handler(environ, start_response)
            return ['']
        err = _HTTP405(sorted(self.method_table.keys()))
        return err.make_wsgi_response(environ, start_response)

## Guide to help in understanding
# @method_dispatcher(*args) -> returns a method_dispatcher_wrapper
#
# @method_dispatcher(*args)
# def func(): pass  -> returns a service_dispatcher_decorator

# service_dispatcher_decorator.method(*args) -> returns 
#    a service_dispatch_decorator_method_wrapper
#
# service_dispatcher_decorator.method(*args)
# def method_func(): pass --> returns a method_wrapper which calls method_func
#
# service_dispatcher_decorator.simple_method(*args)
# def method_func(): pass --> returns a method_wrapper which calls method_func

# This is the top-level decorator
def method_dispatcher(service_id, path=None, wsgi_wrapper=None, query_template=None):
    """Add an Akara resource which dispatches to other functions based on the HTTP method
    
    Used for resources which handle, say, both GET and POST requests.

      service_id - a string which identifies this service; should be a URL
      path - the local URL path to the resource (must not at present
           contain a '/') If None, use the function's name as the path.
      wsgi_wrapper - An outer WSGI component to be wrapped around the methods
      query_template - An Akara URL service template (based on OpenSource; see akara.opensource)
           Can be used to help consumers compose resources withing this service.  The same
           template is used for all HTTP methods

    Example of use:

      @method_dispatcher("http://example.com/example_service")
      def something():
          '''docstring used for the service'''

      @something.simple_method(method="GET", content_type="text/plain",
                               allow_repeated_args=True)
      def something_get(names=[]):
          return "Hi " + ", ".join(names) + "!\n"

      @something.method("POST")
      def something_post(environ, start_response):
          start_response("200 OK", [("Content-Type", "image/gif")])
          return image_bytes

    If you have curl installed then you could access the GET option as:
        curl http://localhost:8880/something?name=Andrew&name=Sara+Marie
    and access the POST option as:
        curl --data "" http://localhost:8880/something

    """
    _no_slashes(path)
    def method_dispatcher_wrapper(func):
        # Have to handle a missing docstring here as otherwise
        # the registry will try to get it from the dispatcher.
        doc = inspect.getdoc(func) or ""
        pth = path
        if pth is None:
            pth = func.__name__
        dispatcher = service_method_dispatcher(pth, wsgi_wrapper)
        registry.register_service(service_id, pth, dispatcher, doc)
        return service_dispatcher_decorator(dispatcher)
    return method_dispatcher_wrapper


class service_dispatcher_decorator(object):
    """Helper class used by method_dispatcher to add new handlers to the given resource

    You should not need to create this directly. Instead, use 'method_dispatcher'
    """
    def __init__(self, dispatcher):
        self.dispatcher = dispatcher

    def method(self, method, encoding="utf-8", writer="xml"):
        """Register a function as a resource handler for a given HTTP method

          method - the relevant HTTP method
          encoding - Used to convert a returned Unicode string or an Amara tree
              to the bytes used in the HTTP response
          writer - Used to serialize the Amara tree for the HTTP response.
              This must be a name which can be used as an Amara.writer.lookup.

        The decorated function must take the normal WSGI parameters
        (environ, start_response) and it must call start_response with
        all the needed headers, including Content-Type.  The function
        may return an Akara tree or a Unicode string, in which case it
        it serialized and converted to bytes based in the 'writer' and
        'encoding' options.
        """
        _check_is_valid_method(method)
        def service_dispatch_decorator_method_wrapper(func):
            @functools.wraps(func)
            def method_wrapper(environ, start_response):
                # 'method' passes the WSGI request straight through
                # to the handler so there's almost no point in
                # setting up the environment. However, I can conceive
                # of tools which might access 'environ' directly, and
                # I want to be consistent with the simple* interfaces.
                new_request(environ)
                result = func(environ, start_response)
                
                # You need to make sure you sent the correct content-type!
                result, ctype, clength = convert_body(result, None, encoding, writer)
                return result

            #For purposes of inspection (not a good idea to change these otherwise you'll lose sync with the values closed over)
            method_wrapper.content_type = None
            method_wrapper.encoding = encoding
            method_wrapper.writer = writer

            self.dispatcher.add_handler(method, method_wrapper)
            return method_wrapper
        return service_dispatch_decorator_method_wrapper

    def simple_method(self, method, content_type=None,
                      encoding="utf-8", writer="xml", allow_repeated_args=False):
        _check_is_valid_method(method)
        if method not in ("GET", "POST"):
            raise ValueError(
                "simple_method only supports GET and POST methods, not %s" %
                (method,))
        
        def service_dispatch_decorator_simple_method_wrapper(func):
            @functools.wraps(func)
            def simple_method_wrapper(environ, start_response):
                try:
                    args, kwargs = _get_function_args(environ, allow_repeated_args)
                except _HTTPError, err:
                    return err.make_wsgi_response(environ, start_response)
                new_request(environ)
                result = func(*args, **kwargs)

                result, ctype, clength = convert_body(result, content_type, encoding, writer)
                send_headers(start_response, ctype, clength)
                return result

            #For purposes of inspection (not a good idea to change these otherwise you'll lose sync with the values closed over)
            simple_method_wrapper.content_type = content_type
            simple_method_wrapper.encoding = encoding
            simple_method_wrapper.writer = writer

            self.dispatcher.add_handler(method, simple_method_wrapper)
            return simple_method_wrapper
        return service_dispatch_decorator_simple_method_wrapper

    # XXX Idea
    #def xml_method(self, method="POST", content_type="text/xml"):
    # ...

# Install some built-in services
@simple_service("GET", "http://purl.org/xml3k/akara/services/registry", "",
                allow_repeated_args=False)
def list_services(service=None):
    return registry.list_services(ident=service) # XXX 'ident' or 'service' ?




# -*- coding: iso-8859-1 -*-
# 
"""
@ 2009 by Uche ogbuji <uche@ogbuji.net>

This file is part of the open source Akara project,
provided under the Apache 2.0 license.
See the files LICENSE and NOTICE for details.
Project home, documentation, distributions: http://wiki.xml3k.org/Akara

 Module name:: moinrest

= Defined REST entry points =

http://purl.org/akara/services/demo/collection (moin)
  - Under there top mount point is oen or more lower points depending on config,
    each of which handles HEAD, GET, POST, PUT, DELETE

= Configuration =

You'll need a config entry such as:

[moinrest]
target-xml3k=http://wiki.xml3k.org

class moinrest:
    targets = {"xml3k": "http://wiki.xml3k.org"}

= Notes on security and authentication =

There are two separate aspects to authentication that moinrest has to
consider and which may need to be configured independently.  First, if
the HTTP server that is running the target wiki is configured with
site-wide basic authentication, you will need to include an
appropriate username and password in the target configuration above.
For example:

[moinrest]
target-xml3k=http://user:password@wiki.xml3k.org

where "user" and "password" are filled in with the appropriate
username and password.  If you're not sure if you need this, try
connecting to the wiki using a browser.  If the browser immediately
displays a pop-up window asking you for a username and password,
you'll need to supply that information in the moinrest configuration
as shown.  If no pop-up window appears, the HTTP server is not using
authentication.

The second form of authentication concerns access to the MoinMoin wiki
itself. In order to modify pages, users may be required to log in to
the wiki first using the wiki's "login" link.  These credentials are
passed to moinrest using HTTP Basic Authentication.  Thus, they need
to be passed in the HTTP headers of requests.  For example, using curl
you would type something like this:

    curl -u me:passwd -p --request PUT --data-binary "@wikicontent.txt" --header "Content-Type: text/plain" "http://localhost:8880/moin/xml3k/FooTest"

Keep in mind that username and password credentials given to moinrest
requests are only for the target wiki.  They are not the same as basic
authentication for the HTTP server hosting the wiki.
"""

#Detailed license and copyright information: http://4suite.org/COPYRIGHT

from __future__ import with_statement

SAMPLE_QUERIES_DOC = '''
Some sample queries:
    curl http://localhost:8880/moin/xml3k/FrontPage
    curl -H "Accept: application/docbook+xml" http://localhost:8880/moin/xml3k/FrontPage
    curl -H "Accept: application/rdf+xml" http://localhost:8880/moin/xml3k/FrontPage
    curl -H "Accept: application/x-moin-attachments+xml" http://localhost:8880/moin/xml3k/FrontPage
    curl --request PUT --data-binary "@wikicontent.txt" --header "Content-Type: text/plain" "http://localhost:8880/moin/xml3k/FooTest"
    curl --request POST --data-binary "@wikicontent.txt" --header "Content-Type: text/plain" "http://localhost:8880/moin/xml3k/FooTest;attachment=wikicontent.txt"
    curl --request DELETE http://localhost:8880/moin/xml3k/FrontPage

    curl -u me:passwd -p --request PUT --data-binary "@wikicontent.txt" --header "Content-Type: text/plain" "http://localhost:8880/moin/xml3k/FooTest"

    Get an attached page:
    curl "http://localhost:8880/moin/xml3k/FooTest;attachment=wikicontent.txt"

    Get a page's history:
    curl http://localhost:8880/moin/xml3k/FrontPage;history
''' #'  # work-around emacs' inability to parse this level of embedded quotes

__doc__ += SAMPLE_QUERIES_DOC

# Standard library imports
import sys     # Used only from sys.stderr
import os
import cgi
import httplib, urllib, urllib2
from string import Template
from cStringIO import StringIO
import tempfile
from contextlib import closing
from wsgiref.util import shift_path_info, request_uri
from functools import wraps
from itertools import dropwhile

# Amara Imports
import amara
from amara import bindery
from amara.writers.struct import structencoder, E, NS, ROOT, RAW
from amara.bindery.html import parse as htmlparse
from amara.bindery.model import examplotron_model, generate_metadata
from amara.lib.util import first_item
from amara.lib.iri import absolutize, relativize, join
from amara.lib.iri import split_fragment, split_uri_ref, unsplit_uri_ref, split_authority
#from amara import inputsource

# Akara Imports
from akara import module_config, logger, response
from akara.util import multipart_post_handler, wsgibase, http_method_handler, copy_headers_to_dict
from akara.services import method_dispatcher
from akara.util import status_response, read_http_body_to_temp
from akara.util import BadTargetError, HTTPAuthorizationError, MoinAuthorizationError, UnexpectedResponseError, MoinMustAuthenticateError, MoinNotFoundError, ContentLengthRequiredError, GenericClientError, ConflictError
import akara.util.moin as moin

# ======================================================================
#                         Module Configruation
# ======================================================================

TARGET_WIKIS = module_config().get("targets", {})
TARGET_WIKI_OPENERS = {}
DEFAULT_OPENER = urllib2.build_opener(
    urllib2.HTTPCookieProcessor(),
    multipart_post_handler.MultipartPostHandler)

# Specifies the default max-age of Moin pages
CACHE_MAX_AGE = module_config().get("CACHE_MAX_AGE", None)

# Specifies a Wiki path (currently only one, FIXME) under which no caching will occur
NO_CACHE_PATHS = module_config().get("NO_CACHE_PATHS", None)

# Look at each Wiki URL and build an appropriate opener object for retrieving
# pages.   If the URL includes HTTP authentication information such as
# http://user:pass@somedomain.com/mywiki, the opener is built with
# basic authentication enabled.   For details, see:
# 
#     : HTTP basic auth: http://www.voidspace.org.uk/python/articles/urllib2.shtml#id6
for k, v in TARGET_WIKIS.items():
    #The target wiki base URI must end in '/'
    v = v.rstrip('/') + '/'
    (scheme, authority, path, query, fragment) = split_uri_ref(v)
    auth, host, port = split_authority(authority)
    authority = host + ':' + port if port else host
    schemeless_url = authority + path
    if auth:
        TARGET_WIKIS[k] = unsplit_uri_ref((scheme, authority, path, query, fragment))
        auth = auth.split(':')
        password_mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
        # Not setting the realm for now, so use None
        password_mgr.add_password(None, scheme+"://"+host+path, auth[0], auth[1])
        password_handler = urllib2.HTTPBasicAuthHandler(password_mgr)
        TARGET_WIKI_OPENERS[k] = urllib2.build_opener(
            password_handler,
            urllib2.HTTPCookieProcessor(),
            multipart_post_handler.MultipartPostHandler)
    else:
        TARGET_WIKI_OPENERS[k] = DEFAULT_OPENER

SERVICE_ID = 'http://purl.org/xml3k/akara/services/demo/moinrest'
DEFAULT_MOUNT = 'moin'

# ======================================================================
#                             Response Templates
# ======================================================================
# These template strings contain the output produced for various
# error conditions.

error_badtarget = Template("""\
404 Not Found

The requested URL $fronturl not found.  
Nothing is known about moin target $target.
"""
)

error_httpforbidden = Template("""\
403 Forbidden

Request for URL $url
is being rejected by the Moin HTTP server due to bad HTTP
authentication. Check the Akara's moinrest configuration to make
sure it includes an appropriate HTTP user name and password.
"""
)

error_moinauthforbidden = Template("""\
403 Forbidden

Request for login URL $url
is being rejected by MoinMoin because the username and password
aren't recognized.  Check your request to moinrest to make sure
a valid Moin username and password are being supplied.
"""
)

error_moinmustauthenticateresponse = Template("""\
401 Unauthorized

Request for URL $url
requires a valid Moin username and password.
"""
)

error_unexpectedresponse = Template("""\
500 Internal Error

Request for URL $url 
failed because an unexpected HTTP status code $code was received.
$error
"""
)

error_moinnotfoundresponse = Template("""\
404 Not Found

The requested URL $fronturl not found.
The URL $backurl was not found in the target wiki.
"""
)

error_contentlengthrequired = Template("""\
411 Length Required

A POST or PUT request was made, but no data was found.
""")

# ======================================================================
#                          moin_error_handler
# ======================================================================
# This error handling function is what actually runs all of the WSGI
# functions implemented by the modules. It catches MoinRest specific exceptions 
# and produces appropriate error responses as needed.
#
# The reason for putting this functionality in a single function is to avoid a lot
# excessive code duplication between different HTTP methods.  For example,
# the handlers for each HTTP method are going to have to deal with
# many of the same error conditions, faults, and responses.  Centralizing
# the handling makes it possible to deal all of the errors in just one place.

def moin_error_wrapper(wsgiapp):
    @wraps(wsgiapp)
    def handler(environ, start_response):
        status_info = {}          # Dictionary of collected status information

        # Replacement for the WSGI start_response function.  This merely
        # collects response data in a dictionary for later use if no errors occur
        def local_start_response(status, headers):
            status_info['status'] = status
            status_info['headers'] = headers

        # Try to run the supplied WSGI handler
        try:
            body = wsgiapp(environ, local_start_response)

            # If control reaches here, no errors.  Proceed with normal WSGI response
            start_response(status_info['status'],status_info['headers'])
            return body

        # Error handling for specifying an invalid moin target name (i.e., not configured, misspelled)
        except BadTargetError,e:
            start_response(status_response(httplib.NOT_FOUND), [
                    ('Content-Type','text/plain')
                    ])
            return error_badtarget.safe_substitute(e.parms)

        # Error handling for back-end HTTP authorization failure.  For example,
        # if the HTTP server hosting MoinMoin has rejected our requests due to
        # bad HTTP authorization.
        except HTTPAuthorizationError,e:
            start_response(status_response(httplib.FORBIDDEN), [
                    ('Content-Type','text/plain')
                    ])
            return error_httpforbidden.safe_substitute(e.parms)

        # Error handling for MoinMoin authorization failure.  This occurs
        # if the user and password supplied to MoinMoin is rejected.
        except MoinAuthorizationError,e:
            start_response(status_response(httplib.FORBIDDEN), [
                    ('Content-Type','text/plain')
                    ])
            return error_moinauthforbidden.safe_substitute(e.parms)

        # Error handling for unexpected HTTP status codes
        except UnexpectedResponseError,e:
            start_response(status_response(httplib.INTERNAL_SERVER_ERROR), [
                    ('Content-Type','text/plain')
                    ])
            return error_unexpectedresponse.safe_substitute(e.parms)

        # Authentication required by MoinMoin.  This isn't an error, but we
        # have to translate this into a 401 response to send back to the client
        # in order to get them to supply the appropriate username/password
        except MoinMustAuthenticateError,e:
            start_response(status_response(httplib.UNAUTHORIZED), [
                    ('Content-Type','text/plain'),
                    ('WWW-Authenticate','Basic realm="%s"' % e.parms.get('target',''))
                    ])
            return error_moinmustauthenticateresponse.safe_substitute(e.parms)
        
        # Page in the target-wiki not found. 404 the client
        except MoinNotFoundError,e:
            start_response(status_response(httplib.NOT_FOUND), [
                    ('Content-Type','text/plain'),
                    ])
            return error_moinnotfoundresponse.safe_substitute(e.parms)

        # Content-length is required for uploaded data
        except ContentLengthRequiredError,e:
            start_response(status_response(httplib.LENGTH_REQUIRED), [
                    ('Content-Type','text/plain')
                    ])
            return error_contentlengthrequired.safe_substitute(e.parms)

        # Used for Moin errors indicated in 2xx HTML responses.  No
        # need for canned text since the error text is in the HTML
        except GenericClientError,e:
            start_response(status_response(httplib.BAD_REQUEST), [
                    ('Content-Type','text/plain')
                    ])
            return e.parms.get('error')

        # For editing (PUT) conflicts detected in 2xx responses
        except ConflictError,e:
            start_response(status_response(httplib.CONFLICT), [
                    ('Content-Type','text/plain')
                    ])
            return e.parms.get('error')

    return handler


# ----------------------------------------------------------------------
#                   Support functions used by handlers
# ----------------------------------------------------------------------

# Utility function for generating status rsponses for WSGI
def status_response(code):
    return '%i %s'%(code, httplib.responses[code])


# Returns information about the target wiki. Raises BadTargetError if nothing
# is known about the target name
def target(environ):
    wiki_id = shift_path_info(environ)
    full_incoming_request = request_uri(environ)
    if wiki_id not in TARGET_WIKIS:
        raise BadTargetError(fronturl=request_uri(environ), target=wiki_id)
    original_page = join(TARGET_WIKIS[wiki_id].rstrip('/')+'/', environ['PATH_INFO'].lstrip('/'))
    #relative_to_wrapped = relativize(, full_incoming_request)
    wrapped_wiki_base = full_incoming_request[:-len(environ['PATH_INFO'])]
    return wiki_id, TARGET_WIKIS[wiki_id], TARGET_WIKI_OPENERS.get(wiki_id), original_page, wrapped_wiki_base


# Check authentication of the user on the MoinMoin wiki
def check_auth(environ, start_response, base, opener, headers=None):
    '''
    Warning: mutates environ in place
    
    If HTTP auth succeeds will also attach a cookie to the opener object in place
    '''
    auth = environ.get('HTTP_AUTHORIZATION')
    #logger.debug('GRIPPO ' + repr((headers)))
    if not auth: 
        return False

    scheme, data = auth.split(None, 1)
    if scheme.lower() != 'basic':
        raise RuntimeError('Unsupported HTTP auth scheme: %s'%scheme)
    username, password = data.decode('base64').split(':', 1)
    url = absolutize('?action=login&name=%s&password=%s&login=login'%(username, password), base)
    request = urllib2.Request(url, None, headers)
    try:
        with closing(opener.open(request)) as resp:
            #Don't need to do anything with the response.  The cookies will be captured automatically
            pass
    except urllib2.URLError,e:
        if e.code == 401:
            # If we're here, the backend HTTP server has likely rejected our request due to HTTP auth
            raise HTTPAuthorizationError(url=url)
        elif e.code == 403:
            # If we get a forbidden response, we made it to MoinMoin but the user name/pass was rejected
            raise MoinAuthorizationError(url=url)
        else:
            raise UnexpectedResponseError(url=url,code=e.code,error=str(e))

    environ['REMOTE_USER'] = username
    return True


def fill_page_edit_form(page, wiki_id, base, opener, headers=None):
    url = absolutize(page, base)
    request = urllib2.Request(url+"?action=edit&editor=text", None, headers)
    #logger.debug('GRIPPO ' + repr((headers)))
    try:
        with closing(opener.open(request)) as resp:
            x = resp.read(); resp = x
            doc = htmlparse(resp)
            raise_embedded_error(doc)

    except urllib2.URLError,e:
        # Comment concerning the behavior of MoinMoin.  If an attempt is made to edit a page 
        # and the user is not authenticated, you will either get a 403 or 404 error depending
        # on whether or not the page being edited exists or not.   If it doesn't exist, 
        # MoinMoin sends back a 404 which is misleading.   We raise MoinMustAuthenticateError
        # to signal the error wrapper to issue a 401 back to the client
        
        #Note: Moin for somereason seems to give 403 errors on some URLs in response to Curl's UA
        if e.code == 403 or e.code == 404:
            raise MoinMustAuthenticateError(url=request.get_full_url(),target=wiki_id)
        else:
            raise UnexpectedResponseError(url=request.get_full_url(),code=e.code,error=str(e))

    try:
        form = doc.html.body.xml_select(u'.//*[@id="editor"]')[0]
    except Exception as ex:
        #XXX There seems to be a crazy XPath bug that only manifests here
        #Use non-XPath as a hack-around :(
        #open('/tmp/foo.html', 'w').write(x)
        logger.debug('Stupid XPath bug.  Working around... ' + repr(ex))
        from amara.lib.util import element_subtree_iter
        form = [ e for e in element_subtree_iter(doc.html.body) if e.xml_attributes.get(u'id') == u'editor' ][0]
        #logger.debug('GRIPPO ' + repr(doc.html.body.xml_select(u'.//form')))
        #logger.debug('GRIPPO ' + repr((form.xml_namespace, form.xml_local, form.xml_qname, form.xml_name, dict(form.xml_attributes))))
        form_vars = {}
        #form / fieldset / input
        form_vars["action"] = [ e for e in element_subtree_iter(form) if e.xml_attributes.get(u'name') == u'action' ][0].xml_attributes[u'value']
        form_vars["rev"] = [ e for e in element_subtree_iter(form) if e.xml_attributes.get(u'name') == u'rev' ][0].xml_attributes[u'value']
        form_vars["ticket"] = [ e for e in element_subtree_iter(form) if e.xml_attributes.get(u'name') == u'ticket' ][0].xml_attributes[u'value']
        form_vars["editor"] = [ e for e in element_subtree_iter(form) if e.xml_attributes.get(u'name') == u'editor' ][0].xml_attributes[u'value']
        #logger.debug('Edit form vars ' + repr(form_vars))
        return form_vars
    form_vars = {}
    #form / fieldset / input
    form_vars["action"] = unicode(form.xml_select(u'string(*/*[@name="action"]/@value)'))
    form_vars["rev"] = unicode(form.xml_select(u'string(*/*[@name="rev"]/@value)'))
    form_vars["ticket"] = unicode(form.xml_select(u'string(*/*[@name="ticket"]/@value)'))
    form_vars["editor"] = unicode(form.xml_select(u'string(*/*[@name="editor"]/@value)'))
    #logger.debug('Edit form vars ' + repr(form_vars))
    return form_vars


def fill_attachment_form(page, attachment, wiki_id, base, opener, headers=None):
    url = absolutize(page, base)
    request = urllib2.Request(url + '?action=AttachFile', None, headers)
    try:
        with closing(opener.open(request)) as resp:
            doc = htmlparse(resp)
            raise_embedded_error(doc)

    except urllib2.URLError,e:
        # Comment concerning the behavior of MoinMoin.  If an attempt is made to post to a page 
        # and the user is not authenticated, you will either get a 403 or 404 error depending
        # on whether or not the page being edited exists or not.   If it doesn't exist, 
        # MoinMoin sends back a 404 which is misleading.   We raise MoinMustAuthenticateError
        # to signal the error wrapper to issue a 401 back to the client
        if e.code == 403 or e.code == 404:
            raise MoinMustAuthenticateError(url=request.get_full_url(),target=wiki_id)
        else:
            raise UnexpectedResponseError(url=request.get_full_url(),code=e.code,error=str(e))

    form = doc.html.body.xml_select(u'.//*[@id="content"]/form')[0]
    form_vars = {}
    #form / dl / ... dd
    #Was called rename in 1.8.x, target in 1.9.x
    form_vars["rename"] = unicode(attachment)
    form_vars["target"] = unicode(attachment)
    #FIXME: parameterize
    form_vars["overwrite"] = u'1'
    form_vars["action"] = unicode(form.xml_select(u'string(*/*[@name="action"]/@value)'))
    form_vars["do"] = unicode(form.xml_select(u'string(*/*[@name="do"]/@value)'))
    form_vars["ticket"] = unicode(form.xml_select(u'string(*/*[@name="ticket"]/@value)'))
    form_vars["submit"] = unicode(form.xml_select(u'string(*/*[@type="submit"]/@value)'))
    #pprint.pprint(form_vars)
    return form_vars


def fill_page_delete_form(page, wiki_id, base, opener, headers=None):
    url = absolutize(page, base)
    request = urllib2.Request(url+"?action=DeletePage", None, headers)
    try:
        with closing(opener.open(request)) as resp:
            x = resp.read(); resp = x
            doc = htmlparse(resp)
            raise_embedded_error(doc)

    except urllib2.URLError,e:
        if e.code == 403:
            raise MoinMustAuthenticateError(url=request.get_full_url(),target=wiki_id)
        else:
            raise UnexpectedResponseError(url=request.get_full_url(),code=e.code,error=str(e))

    form = doc.html.body.xml_select(u'.//form[@method="POST"]')[0]
    form_vars = {}
    form_vars["action"] = unicode(form.xml_select(u'string(*/*[@name="action"]/@value)'))
    form_vars["ticket"] = unicode(form.xml_select(u'string(*/*[@name="ticket"]/@value)'))
    form_vars["delete"] = unicode(form.xml_select(u'string(//input[@type="submit" and @name="delete"]/@value)'))
    return form_vars


def scrape_page_history(page, base, opener, headers=None):
    url = absolutize(page, base)
    request = urllib2.Request(url+"?action=info", None, headers)
    try:
        with closing(opener.open(request)) as resp:
            doc = htmlparse(resp)
            raise_embedded_error(doc)

    except urllib2.URLError,e:
        # Comment concerning the behavior of MoinMoin.  If an attempt is made to post to a page 
        # and the user is not authenticated, you will either get a 403 or 404 error depending
        # on whether or not the page being edited exists or not.   If it doesn't exist, 
        # MoinMoin sends back a 404 which is misleading.   We raise MoinMustAuthenticateError
        # to signal the error wrapper to issue a 401 back to the client
        if e.code == 403 or e.code == 404:
            raise MoinMustAuthenticateError(url=request.get_full_url(),target=wiki_id)
        else:
            raise UnexpectedResponseError(url=request.get_full_url(),code=e.code,error=str(e))

    info = []
    try:
        table = doc.html.body.xml_select(u'.//table[@id="dbw.table"]')[0]
    except Exception as ex:
        #XXX Seems to be a crazy XPath bug that only manifests here
        #Use non-XPath as a hack-around :(
        logger.debug('Stupid XPath bug.  Working around... ' + repr(ex))
        from amara.lib.util import element_subtree_iter
        table = [ e for e in element_subtree_iter(doc.html.body) if e.xml_attributes.get(u'id') == u'dbw.table' ]
        if not table:
            #"Revision History... No log entries found." i.e. page not even yet created
            return info
    info = [
        dict(rev=tr.td[0], date=tr.td[1], editor=tr.td[4])
        for tr in table[0].xml_select(u'.//tr[td[@class="column1"]]')
        #for tr in table.tbody.tr if tr.xml_select(u'td[@class="column1"]')
    ]
    return info

# Extract any error embedded in an HTML response (returned by Moin in 2xx responses),
# and raise it as an HTTP error.  Would be nice to handle this generically in
# moin_error_wrapper, but don't want to incur HTML parse cost
#
def raise_embedded_error(doc):

    try:
        edit_textarea = doc.xml_select(u'.//*[@id="editor-textarea"]/text()')
    except:
        pass

    if edit_textarea and 'End of edit conflict' in edit_textarea:
        raise ConflictError(error=edit_textarea)

    try:
        error_div = doc.xml_select('//div[@class="error"]')
    except:
        return

    if error_div:
        raise GenericClientError(error=error_div.asString())


# ----------------------------------------------------------------------
#                       HTTP Method Handlers
# ----------------------------------------------------------------------
# The following functions implement versions of the various HTTP methods 
# (GET, HEAD, POST, PUT, DELETE).  Each method is actually implemented as a
# a pair of functions.  One is a private implementation (e.g., _get_page).  
# The other function is a wrapper that encloses each handler with the error 
# handling function above (moin_error_handler).   Again, this is to avoid
# excessive duplication of error handling code.

@method_dispatcher(SERVICE_ID, DEFAULT_MOUNT, wsgi_wrapper=moin_error_wrapper)
def dispatcher():
    __doc__ = SAMPLE_QUERIES_DOC
    return

@dispatcher.method("GET")
def get_page(environ, start_response):
    #logger.debug('get_page: ' + repr((environ['SCRIPT_NAME'], environ['PATH_INFO'])))
    req_headers = copy_headers_to_dict(environ,exclude=['HTTP_ACCEPT_ENCODING'])
    wiki_id, base, opener, original_page, wrapped_wiki_base = target(environ)
    page = environ['PATH_INFO'].lstrip('/')
    check_auth(environ, start_response, base, opener, req_headers)
    upstream_handler = None
    status = httplib.OK
    params = cgi.parse_qs(environ['QUERY_STRING'])
    #Note: probably a better solution here: http://code.google.com/p/mimeparse/
    accepted_imts = environ.get('HTTP_ACCEPT', '').split(',')
    #logger.debug('accepted_imts: ' + repr(accepted_imts))
    imt = first_item(dropwhile(lambda x: '*' in x, accepted_imts))
    #logger.debug('imt: ' + repr(imt))
    params_for_moin = {}
    cache_max_age = CACHE_MAX_AGE # max-age of this response. If set to None, it will not be used
    if NO_CACHE_PATHS and first_item(dropwhile(lambda x: x not in page, NO_CACHE_PATHS)):
        cache_max_age = None

    if 'rev' in params:
        #XXX: Not compatible with search
        #params_for_moin = {'rev' : params['rev'][0], 'action': 'recall'}
        params_for_moin = {'rev' : params['rev'][0]}
    if 'search' in params:
        searchq = params['search'][0]
        query = urllib.urlencode({'value' : searchq, 'action': 'fullsearch', 'context': '180', 'fullsearch': 'Text'})
        #?action=fullsearch&context=180&value=foo&=Text
        url = absolutize('?'+query, base)
        request = urllib2.Request(url, None, req_headers)
        ctype = moin.RDF_IMT
        cache_max_age = None
    #elif 'action' in params and params['action'][0] == 'recall':
    elif moin.HTML_IMT in environ.get('HTTP_ACCEPT', ''):
        params = urllib.urlencode(params_for_moin)
        url = absolutize(page+'?'+params, base)
        request = urllib2.Request(url, None, req_headers)
        ctype = moin.HTML_IMT
    elif moin.RDF_IMT in environ.get('HTTP_ACCEPT', ''):
        #FIXME: Make unique flag optional
        #url = base + '/RecentChanges?action=rss_rc&unique=1&ddiffs=1'
        url = absolutize('RecentChanges?action=rss_rc&unique=1&ddiffs=1', base)
        #print >> sys.stderr, (url, base, '/RecentChanges?action=rss_rc&unique=1&ddiffs=1', )
        request = urllib2.Request(url, None, req_headers)
        ctype = moin.RDF_IMT
    elif moin.ATTACHMENTS_IMT in environ.get('HTTP_ACCEPT', ''):
        url = absolutize(page + '?action=AttachFile', base)
        request = urllib2.Request(url, None, req_headers)
        ctype = moin.ATTACHMENTS_IMT
        def upstream_handler():
            #Sigh.  Sometimes you have to break some Tag soup eggs to make a RESTful omlette
            with closing(opener.open(request)) as resp:
                rbody = resp.read()
            doc = htmlparse(rbody)
            raise_embedded_error(doc)
            attachment_nodes = doc.xml_select(u'//*[contains(@href, "action=AttachFile") and contains(@href, "do=view")]')
            targets = []
            for node in attachment_nodes:
                target = [ param.split('=', 1)[1] for param in node.href.split(u'&') if param.startswith('target=') ][0]
                targets.append(target)
            output = structencoder(indent=u"yes")
            output.feed(
            ROOT(
                E((u'attachments'),
                    (E(u'attachment', {u'href': unicode(t)}) for t in targets)
                )
            ))
            return output.read(), ctype
    #Notes on use of URI parameters - http://markmail.org/message/gw6xbbvx4st6bksw
    elif ';attachment=' in page:
        page, attachment = page.split(';attachment=', 1)
        url = absolutize(page + '?action=AttachFile&do=get&target=' + attachment, base)
        request = urllib2.Request(url, None, req_headers)
        def upstream_handler():
            with closing(opener.open(request)) as resp:
                rbody = resp.read()
            return rbody, dict(resp.info())['content-type']
    #
    elif ';history' in page:
        cache_max_age = None
        page, discard = page.split(';history', 1)
        ctype = moin.XML_IMT
        def upstream_handler():
            revs = scrape_page_history(page, base, opener, req_headers)
            output = structencoder(indent=u"yes")
            output.feed(
            ROOT(
                E((u'history'),
                    (E(u'rev', {u'id': unicode(r['rev']), u'editor': unicode(r['editor']), u'date': unicode(r['date']).replace(' ', 'T')}) for r in revs)
                )
            ))
            return output.read(), ctype
    elif imt:
        params_for_moin.update({'mimetype': imt})
        params = urllib.urlencode(params_for_moin)
        url = absolutize(page, base) + '?' + params
        request = urllib2.Request(url, None, req_headers)
        ctype = moin.DOCBOOK_IMT
    else:
        params_for_moin.update({'action': 'raw'})
        params = urllib.urlencode(params_for_moin)
        url = absolutize(page, base) + '?' + params
        request = urllib2.Request(url, None, req_headers)
        ctype = moin.WIKITEXT_IMT
    try:
        if upstream_handler:
            rbody, ctype = upstream_handler()
        else:
            with closing(opener.open(request)) as resp:
                rbody = resp.read()
        
        #headers = {moin.ORIG_BASE_HEADER: base}
        #moin_base = absolutize(wiki_id, base)
        moin_base_info = base + ' ' + wrapped_wiki_base + ' ' + original_page
        response_headers = [("Content-Type", ctype),
                            ("Vary", "Accept"),
                            (moin.ORIG_BASE_HEADER, moin_base_info)]
        if cache_max_age:
            response_headers.append(("Cache-Control","max-age="+str(cache_max_age)))

        start_response(status_response(status), response_headers)
        return rbody
    except urllib2.URLError, e:
        if e.code == 401:
            raise HTTPAuthorizationError(url=request.get_full_url())
        if e.code == 403:
            raise MoinMustAuthenticateError(url=request.get_full_url(),target=wiki_id)
        if e.code == 404:
            raise MoinNotFoundError(fronturl=request_uri(environ),backurl=url)
        else:
            raise UnexpectedResponseError(url=url,code=e.code,error=str(e))


# PUT handler
@dispatcher.method("PUT")
def _put_page(environ, start_response):
    '''
    '''
    req_headers = copy_headers_to_dict(environ,exclude=['HTTP_ACCEPT_ENCODING'])
    wiki_id, base, opener, original_page, wrapped_wiki_base = target(environ)
    page = environ['PATH_INFO'].lstrip('/')
    check_auth(environ, start_response, base, opener, req_headers)

    ctype = environ.get('CONTENT_TYPE', 'application/unknown')
    temp_fpath = read_http_body_to_temp(environ, start_response)
    form_vars = fill_page_edit_form(page, wiki_id, base, opener, req_headers)
    form_vars["savetext"] = open(temp_fpath, "r").read()

    url = absolutize(page, base)
    data = urllib.urlencode(form_vars)
    request = urllib2.Request(url, data, req_headers)
    try:
        logger.debug('Prior to urllib2.opener')
        with closing(opener.open(request)) as resp:
            logger.debug('Return from urllib2.opener')
            doc = htmlparse(resp)
            raise_embedded_error(doc)
            logger.debug('HTML parse complete post urllib2.opener')
    except urllib2.URLError,e:
        raise UnexpectedResponseError(url=url,code=e.code,error=str(e))

    wrapped_url = join(wrapped_wiki_base, page)
    msg = 'Page updated OK: %s (%s)'%(url, wrapped_url)
    #response.add_header("Content-Length", str(len(msg)))
    moin_base_info = base + ' ' + wrapped_wiki_base + ' ' + original_page
    headers = [
        ("Content-Type", "text/plain"),
        ("Content-Location", wrapped_url),
        (moin.ORIG_BASE_HEADER, moin_base_info),
        (moin.WIKI_RELATIVE_HEADER, relativize(wrapped_url, wrapped_wiki_base)),
        ]
    start_response(status_response(httplib.CREATED), headers)
    return [msg]


# POST handler
@dispatcher.method("POST")
def post_page(environ, start_response):
    '''
    Attachments use URI path params
    (for a bit of discussion see http://groups.google.com/group/comp.lang.python/browse_thread/thread/4662d41aca276d99)
    '''
    #ctype = environ.get('CONTENT_TYPE', 'application/unknown')

    req_headers = copy_headers_to_dict(environ,exclude=['HTTP_ACCEPT_ENCODING'])
    wiki_id, base, opener, original_page, wrapped_wiki_base = target(environ)
    logger.debug("wiki_id,base,opener,original_age,wrapped_wiki_base="+repr((wiki_id,base,opener,original_page,wrapped_wiki_base)))
    check_auth(environ, start_response, base, opener, req_headers)

    page = environ['PATH_INFO'].lstrip('/')
    page, chaff, attachment = page.partition(';attachment=')
#    print >> sys.stderr, page, attachment
    #now = datetime.now().isoformat()
    #Unfortunately because urllib2's data dicts don't give an option for limiting read length, must read into memory and wrap
    #content = StringIO(environ['wsgi.input'].read(clen))
    temp_fpath = read_http_body_to_temp(environ, start_response)
    form_vars = fill_attachment_form(page, attachment, wiki_id, base, opener, req_headers)
    form_vars["file"] = open(temp_fpath, "rb")

    url = absolutize(page, base)
    #print >> sys.stderr, url, temp_fpath
    #data = urllib.urlencode(form_vars)
    request = urllib2.Request(url, form_vars, req_headers)
    try:
        with closing(opener.open(request)) as resp:
            doc = htmlparse(resp)
            raise_embedded_error(doc)
            #logger.debug('POST for attachment page response... ' + doc.xml_encode())

    except urllib2.URLError,e:
        if e.code == 404:
            raise MoinNotFoundError(fronturl=request_uri(environ), backurl=url)
        else:
            raise UnexpectedResponseError(url=url,code=e.code,error=str(e))

    form_vars["file"].close()
    os.remove(temp_fpath)

    msg = 'Attachment updated OK: %s\n'%(url)

    #response.add_header("Content-Length", str(len(msg)))
    moin_base_info = base + ' ' + wrapped_wiki_base + ' ' + original_page
    headers = [
        ("Content-Type", "text/plain"),
        ("Content-Location", url),
        (moin.ORIG_BASE_HEADER, moin_base_info),
        (moin.WIKI_RELATIVE_HEADER, relativize(url, wrapped_wiki_base)),
        ]
    start_response(status_response(httplib.CREATED), headers)
    return msg

# DELETE handler
@dispatcher.method("DELETE")
def _delete_page(environ, start_response):
    '''
    Deletes a Wiki page, returning 200 if successful.  Does not yet support
    the deletion of attachments.

    '''
    #The Moin form asks that this be in multipart-form format, but the multipart handler
    #fallsback to url-encoding unless you pass it a file.  Luckily, the equivalent
    #url-encoded request works... for now.
    
    req_headers = copy_headers_to_dict(environ,exclude=['HTTP_ACCEPT_ENCODING'])
    wiki_id, base, opener, original_page, wrapped_wiki_base = target(environ)
    page = environ['PATH_INFO'].lstrip('/')
    check_auth(environ, start_response, base, opener, req_headers)

    form_vars = fill_page_delete_form(page, wiki_id, base, opener, req_headers)

    url = absolutize(page, base)

    request = urllib2.Request(url, form_vars, req_headers)
    try:
        with closing(opener.open(request)) as resp:
            doc = htmlparse(resp)
            raise_embedded_error(doc)
    except urllib2.URLError,e:
        
        if e.code == 404:
            # Moin returns 404 on a succcessful DeletePage POST; recast as a 200
            pass
        else:
            raise UnexpectedResponseError(url=url,code=e.code,error=str(e))

    msg = 'Page deleted OK: ' + url
    start_response(status_response(httplib.OK),[("Content-Type","text/plain")])
    return msg



import unittest
from amara.lib import testsupport
import os
import tempfile
import sqlite3
from cStringIO import StringIO
import amara

from akara.resource.repository import driver

MONTY_XML = """<monty>
  <python spam="eggs">What do you mean "bleh"</python>
  <python ministry="abuse">But I was looking for argument</python>
</monty>"""

NS_XML = """<doc xmlns:a="urn:bogus:a" xmlns:b="urn:bogus:b">
  <a:monty/>
  <b:python/>
</doc>"""

#TEST_FILE = "Xml/Core/disclaimer.xml"

### modify this url
#TEST_URL = "http://cvs.4suite.org/viewcvs/*checkout*/4Suite/test/Xml/Core/disclaimer.xml"


class Test_parse_functions_1(unittest.TestCase):
    """Testing local sources"""
    def test_simple_roundtrip(self):
        """Parse with string"""
        def myindex(content):
            doc = amara.parse(content)
            yield u'root-element-name', doc.xml_select(u'name(*)')
            yield u'element-count', doc.xml_select(u'count(//*)')
        fname = tempfile.mktemp('.xml')
        driver.init_db(sqlite3.connect(fname))
        drv = driver(sqlite3.connect(fname))
        content = MONTY_XML
        id = drv.create_resource(content, metadata=dict(myindex(content)))
        content1, metadata = drv.get_resource(id)
        doc = amara.parse(content)
        self.assertEqual(content, content1)
        self.assertEqual(metadata[u'root-element-name'], doc.xml_select(u'name(*)'))
        self.assertEqual(metadata[u'element-count'], doc.xml_select(u'count(//*)'))
        return


if __name__ == '__main__':
    testsupport.test_main()



# -*- encoding: utf-8 -*-
'''
@ 2009 by Uche ogbuji <uche@ogbuji.net>

This file is part of the open source Akara project,
provided under the Apache 2.0 license.
See the files LICENSE and NOTICE for details.
Project home, documentation, distributions: http://wiki.xml3k.org/Akara

 Module name:: echo
 
Responds to POST with the same body content as sent in the request

= Defined REST entry points =

http://purl.org/akara/services/demo/echo (akara.echo) Handles POST

= Configuration =

No configuration required

= Notes on security =

This module only sends information available in the request.  No security implications.
'''

import amara
from akara.services import simple_service

ECHO_SERVICE_ID = 'http://purl.org/xml3k/akara/services/demo/echo'


@simple_service('POST', ECHO_SERVICE_ID, 'akara.echo')
def akara_echo_body(body, ctype, log=u'no'):
    '''
    Sample request:
    curl --request POST --data-binary "@foo.dat" --header "Content-type: text/plain" "http://localhost:8880/akara.echo"
    '''
    if log == u'yes':
        from akara import logger
        logger.debug('akara_echo_body: ' + body)
    return body




# -*- encoding: utf-8 -*-
'''Serve static files from the file system

Requires a configuration section, for example:

class static:
    akara_module = "akara.demo.static"
    paths = {"images": "/path/to/image/directory",
             "~me": "/home/directory/public_files"}

'''

import os
import stat
import mimetypes
from email.utils import formatdate
import warnings

from akara import registry

SERVICE_ID = 'http://purl.org/akara/services/demo/static'

class MediaHandler(object):

    __name__ = 'MediaHandler'

    def __init__(self, media_root):
        media_root = os.path.abspath(media_root)
        if not media_root.endswith(os.sep):
            media_root += os.sep
        self.media_root = media_root

    def __call__(self, environ, start_response):

        path_info = environ['PATH_INFO']
        if path_info[:1] == '/':
            path_info = path_info[1:]

        filename = os.path.join(self.media_root, path_info)
        # Simple security check.
        # Things like "con" on Windows will mess it up.
        filename = os.path.normpath(filename)
        if not filename.startswith(self.media_root):
            start_response('401 Unauthorized', [('Content-Type', 'text/plain')])
            return ["Path is outside of the served directory"]

        if not os.path.exists(filename):
            start_response('404 Not Found', [('Content-Type', 'text/plain')])
            return ['Nothing matches the given URI']

        try:
            fp = open(filename, 'rb')
        except IOError:
            start_response('401 Unauthorized', [('Content-Type', 'text/plain')])
            return ['Permission denied']

        # This is a very simple implementation of conditional GET with
        # the Last-Modified header. It makes media files a bit speedier
        # because the files are only read off disk for the first request
        # (assuming the browser/client supports conditional GET).
        mtime = formatdate(os.stat(filename).st_mtime, usegmt=True)
        headers = [('Last-Modified', mtime)]
        if environ.get('HTTP_IF_MODIFIED_SINCE', None) == mtime:
            status = '304 Not Modified'
            output = ()
        else:
            status = '200 OK'
            mime_type = mimetypes.guess_type(filename)[0]
            if mime_type:
                headers.append(('Content-Type', mime_type))
            output = [fp.read()]
            fp.close()
        start_response(status, headers)
        return output

import akara

if not akara.module_config():
    akara.logger.warn("No configuration section found for %r" % (__name__,))
    
paths = akara.module_config().get("paths", {})

for path, root in paths.items():
    handler = MediaHandler(root)
    registry.register_service(SERVICE_ID, path, handler)



# -*- encoding: utf-8 -*-
'''
'''

from __future__ import with_statement

import sys
import urllib, urllib2, urlparse
from cStringIO import StringIO
from gettext import gettext as _
from unicodedata import *

from amara.bindery import html
#from amara.lib.util import *

from akara.services import simple_service
from amara.writers.struct import *

NAME_REQUIRED = _("The 'name' query parameter is mandatory.")


SERVICE_ID = 'http://purl.org/akara/services/demo/unicode.charbyname'
#@simple_service('get', SERVICE_ID, 'akara.rdfa.json', 'application/json')
@simple_service('GET', SERVICE_ID, 'akara.unicode.charbyname', 'text/plain')
def charbyname(name=None):
    '''
    name - the Unicode character name to look up
    
    Sample request:
    curl "http://localhost:8880/akara.unicode.charbyname?name=DOUBLE+DAGGER"
    '''
    if name is None:
        raise AssertionError(NAME_REQUIRED)
    try:
        return lookup(name).encode('utf-8')
    except KeyError:
        return ""


Q_REQUIRED = _("The 'q' query parameter is mandatory.")
UINFO_SEARCH_URL = u"http://www.fileformat.info/info/unicode/char/search.htm?preview=entity&"

SERVICE_ID = 'http://purl.org/akara/services/demo/unicode.search'
@simple_service('GET', SERVICE_ID, 'akara.unicode.search', 'application/xml')
def charsearch(q=None):
    '''
    name - a string to search for in Unicode information (using http://www.fileformat.info )
    
    Sample request:
    curl "http://localhost:8880/akara.unicode.search?q=dagger"
    '''
    if q is None:
        raise AssertionError(Q_REQUIRED)
    query = urllib.urlencode({"q": q})
    search_url = UINFO_SEARCH_URL + query
    doc = html.parse(search_url)
    buf = StringIO()
    structwriter(indent=u"yes", stream=buf).feed(
    ROOT(
        E((u'characters'),
            (E(u'character', {u'see-also': urlparse.urljoin(search_url, row.td[0].a.href),
                              u'name': unicode(row.td[2]) },
               unicode(row.td[3]))
             for row in doc.xml_select(u'//*[@class="list"]//*[starts-with(@class, "row")]'))
        )
    ))
    return buf.getvalue()




"""Service registry

Add WSGI handlers to the Akara HTTP dispatch registry.

"""

import inspect

import amara
from amara import tree

from akara import logger

# Take care! This is only initalized when the server starts up and
# after it reads the config file. It's used to generate the full
# template name in list_services().
from akara import global_config

from akara import opensearch

__all__ = ("register_service", "get_service")

#### Simple registry of services


# We had some discussion about using the term 'path' or 'mount_point'?
# Right now we don't really have '/' paths so there's no real difference
# Therefore, you register mount_points, which must not have a "/" in them.
# Incoming requests have a path, and the first segment (before the "/")
# is used to find the mount point.

# We're calling it a 'path' for future compatibility.  For more
# complex paths we might want to use Routes or similar system, in
# which case we'll also have pattern matching on the path segments.
# You'll likely register a path pattern, still with the name 'path'.

class Service(object):
    "Internal class to store information about a given service resource"
    def __init__(self, handler, path, ident, doc, query_template):
        self.handler = handler # most important - the function to call
        # XXX is it okay for the path to be None? I think so ...
        self.path = path  # where to find the service
        self.ident = ident  # URN which identifies this uniquely
        self.doc = doc  # description to use when listing the service
        self.query_template = query_template # OpenSearch template fragment
        self._template = None # The OpenSearch Template
        self._internal_template = None # The OpenSearch template used internally

    @property
    def template(self):
        if self._template is False:
            # No template is possible
            return None
        if self._template is not None:
            # Cached version
            return self._template
        # Compute the template
        if self.query_template is None:
            # No template available
            self._template = False
            return None
        template = global_config.server_root + self.query_template
        self._template = opensearch.make_template(template)
        return self._template

    @property
    def internal_template(self):
        if self._internal_template is False:
            # No template is possible
            return None
        if self._internal_template is not None:
            # Cached version
            return self._internal_template
        # Compute the template
        if self.query_template is None:
            # No template available
            self._internal_template = False
            return None
        internal_template = global_config.internal_server_root + self.query_template
        self._internal_template = opensearch.make_template(internal_template)
        return self._internal_template
    

class Registry(object):
    "Internal class to handle resource registration information"
    def __init__(self):
        self._registered_services = {}

    def register_service(self, ident, path, handler, doc=None, query_template=None):
        if "/" in path:
            raise ValueError("Registered path %r may not contain a '/'" % (path,))
        if doc is None:
            doc = inspect.getdoc(handler) or ""
        if ident in self._registered_services:
            logger.warn("Replacing mount point %r (%r)" % (path, ident))
        else:
            logger.debug("Created new mount point %r (%r)" % (path, ident))
        serv = Service(handler, path, ident, doc, query_template)
        self._registered_services[path] = serv

    def get_service(self, path):
        return self._registered_services[path]

    def list_services(self, ident=None):
        document = tree.entity()
        services = document.xml_append(tree.element(None, 'services'))
        for path, service in sorted(self._registered_services.iteritems()):
            if ident is not None and service.ident != ident:
                continue
            service_node = services.xml_append(tree.element(None, 'service'))
            service_node.xml_attributes['ident'] = service.ident
            E = service_node.xml_append(tree.element(None, 'path'))
            template = service.template
            if template is not None:
                E.xml_attributes["template"] = service.template.template
            E.xml_append(tree.text(path))
            E = service_node.xml_append(tree.element(None, 'description'))
            E.xml_append(tree.text(service.doc))
        return document

_current_registry = Registry()


def register_service(ident, path, function, doc=None, query_template=None):
    _current_registry.register_service(ident, path, function, doc, query_template)

def get_service(mount_point):
    return _current_registry.get_service(mount_point)

def list_services(ident=None):
    return _current_registry.list_services(ident)

def get_a_service_by_id(ident):
    for path, service in _current_registry._registered_services.items():
        if service.ident == ident:
            return service
    return None


# ident -> template
_registered_templates = {}

# Split this up to make it easier to test
def _register_services(uri):
    new_templates = {}
    tree = amara.parse(uri)
    for path in tree.xml_select("//service[@ident]/path[@template]"):
        ident = path.xml_parent.xml_attributes["ident"]
        template = path.xml_attributes["template"]
        new_templates[ident] = opensearch.make_template(template)
    return new_templates

def register_services(uri):
    new_templates = _register_services(uri)
    _registered_templates.update(new_templates)

def register_template(ident, template):
    if isinstance(template, basestring):
        template = opensearch.make_template(template)
    _registered_templates[ident] = template

def _get_url(ident, template_attr, kwargs):
    # Get the local service first
    service = get_a_service_by_id(ident)
    if service is not None:
        template = getattr(service, template_attr)
    else:
        # Still not here? Look for the other registered templates.
        template = _registered_templates.get(ident, None)

    if template is None:
            # XXX What's a good default? Just put them as kwargs at the end?
            raise TypeError("service %r does not have a query template" % (ident,))
    return template.substitute(**kwargs)

def get_service_url(ident, **kwargs):
    return _get_url(ident, "template", kwargs)

def get_internal_service_url(ident, **kwargs):
    return _get_url(ident, "internal_template", kwargs)



"""Support code for dealing with distutils and (eventually) Distribute.

This is a helper module for installing Akara extensions. It extends
the standard distutils installer. Please read 
  http://docs.python.org/distutils/
for a detailed description of how to use and distribute software
packages with distutils.

This version of setup() supports the "akara_extensions" option, which
is a list of Python files to install into the Akara extension module
directory. Your setup.py file might look like this:

  from akara.dist import setup

  setup(name = "MyAkaraExtension",
        version = "9.8",
        description = "An example Akara extension",
        akara_extensions = ["module1.py", "module2.py"],
        )

and here is an example of it in use:

  % python setup.py install


All of the normal distutils options, like the ability to install
Python modules, C-based extensions, and data files, are still
available. This does mean that all packages will create an egg, even
if no other Python code is installed.

The Akara extension directory is not a versioned directory. Extensions
placed in that directory are not managed by the Python egg system and
will not work under virtualenv or similar tools.

The default extension directory location comes from the default Akara
configuration file. You can specify an alternate configuration file
with --akara-config, as in

  % python setup.py install --akara-config ~/my_akara.conf

or specify the installation directory directly with
--akara-modules-dir, as in

  % python setup.py install --akara-modules-dir ~/local/modules

"""
import os
import shutil

from distutils.core import setup as _setup
from distutils.core import Command
from distutils import log

# Has not been tested against setuptools or Distribute and probably
# won't work with them.
# Let the Akara development team know if that's a problem.

from distutils.command.install import install as old_install
from distutils.dist import Distribution as _Distribution

import akara.read_config


MISSING_AKARA_EXTENSIONS = (
"If you call the Akara setup adapter for distutils.core.setup then\n"
"you need to include the 'akara_extensions' parameter, which is a\n"
"list of one or more filenames ending with '.py'")

EXTENSION_OPTIONS = [
    ("akara-config=", None,
     ("Location of an existing akara.conf use to get the 'modules' directory "
      "if --akara-modules-dir is not specified")),
    ("akara-modules-dir=", None,
     ("Directory in which to place the new Akara extension and configuration files. "
      "If not specified, get it from akara.conf")),
    ]


def setup(**kwargs):
    if "akara_extensions" not in kwargs:
        raise SystemExit(MISSING_AKARA_EXTENSIONS)
    for filename in kwargs["akara_extensions"]:
        if not filename.endswith(".py"):
            raise SystemExit(
                "Akara extensions must end with '.py' not %r" %
                (filename,))
    for filename in kwargs.get("akara_extension_confs", []):
        if not filename.endswith(".conf"):
            raise SystemExit(
                "Akara extension configuration files must end with '.conf' not %r" %
                (filename,))

    new_kwargs = kwargs.copy()

    # Create our new installation code.
    # Figure out which command class to wrap.
    cmdclass = new_kwargs.get('cmdclass', {})
    if 'install' in cmdclass:
        install = cmdclass['install']
    else:
        install = old_install

    # A hook to add our own extensions
    class my_install(install):
        sub_commands = install.sub_commands + [
                ('install_akara_extensions', None)
        ]
        user_options = install.user_options + EXTENSION_OPTIONS
        def initialize_options(self):
            install.initialize_options(self)
            self.akara_config = None
            self.akara_modules_dir = None


    # Our installation extension
    class install_akara_extensions(Command):
        description = "Command to install akara extensions"

        user_options = EXTENSION_OPTIONS

        def initialize_options(self):
            # I so don't know what I'm doing here, but it seems to work.
            args =  self.distribution.command_options["install"]
            self.akara_modules_dir = self.akara_config = None
            for key, value in args.items():
                if key == "akara_modules_dir":
                    self.akara_modules_dir = value[1]
                elif key == "akara_config":
                    self.akara_config = value[1]

        def finalize_options(self):
            if self.akara_modules_dir is None:
                settings, config = akara.read_config.read_config(self.akara_config)
                self.akara_modules_dir = settings["module_dir"]

        def run (self):
            dist = self.distribution
            for (description, filenames) in (
                ("extension", dist.akara_extensions),
                ("configuration", dist.akara_extension_confs)):
                for filename in filenames:
                    log.info("Installing Akara %s %r in %r" %
                             (description, filename, self.akara_modules_dir))
                    if not self.dry_run:
                        if not os.path.isdir(self.akara_modules_dir):
                            os.makedirs(self.akara_modules_dir) # Make sure the directory exists
                        shutil.copy(filename, self.akara_modules_dir)

    new_cmdclass = {}
    new_cmdclass.update(cmdclass)
    new_cmdclass['install'] = my_install
    new_cmdclass['install_akara_extensions'] = install_akara_extensions
    new_kwargs['cmdclass'] = new_cmdclass

    # Handle overriden distclass
    if 'distclass' in new_kwargs:
        Distribution = new_kwargs['distclass']
    else:
        Distribution = _Distribution
    class MyDistribution(Distribution):
        def __init__(self, attrs=None):
            for opt in ("akara_extensions",
                        "akara_extension_confs"):
                assert not hasattr(self, opt)
                setattr(self, opt, attrs.get(opt, []))

            Distribution.__init__(self, attrs)

    new_kwargs['distclass'] = MyDistribution
    return _setup(**new_kwargs)



# -*- coding: utf-8 -*-
"""
Akara transform middleware (XSLT and more)

XSLT:

xslt_transform_manager manages a pool of Amara XSLT processor instances
to dispatch requests rapidly.

Note on extensions:

If you wish to use any XPath or XSLT extensions put the extension modules
into a directory and indicate that directory in your server config such
that it's made available as environ['http://purl.org/xml3k/akara/transform/extensions'].
An example:

-- % --
... = '/usr/share/amara/extensions'
-- % --

For more on Amara XSLT extensions See <http://4suite.org/docs/CoreManual.xml> <- FIXME: update
"""

# Used for locking the cache of XSLT processors
import thread
# Used to check whether or not an XSLT stylesheet has been modified
from stat import ST_MTIME
from os import stat

import amara
from amara.lib import iri
from amara.lib import inputsource
from amara.xpath.util import parameterize
from amara.xslt.processor import processor
from amara import sax

USER_AGENT_REGEXEN = [
'.*MSIE 5.5.*',
'.*MSIE 6.0.*',
'.*MSIE 7.0.*',
'.*Gecko/2005.*',
'.*Gecko/2006.*',
'.*Opera/9.*',
'.*AppleWebKit/31.*',
'.*AppleWebKit/4.*',
]

USER_AGENT_REGEXEN = [ re.compile(regex) for regex in USER_AGENT_REGEXEN ]
WSGI_NS = u'http://www.wsgi.org/'

MTYPE_PAT = re.compile('.*/.*xml.*')

#We use processor pooling, but we eventually want to get it to the point where we can just
#cache compiled stylesheets

#FIXME: does not yet handle extensions

DUMMY_SOURCE_DOC_URI = "http://purl.org/xml3k/akara/transform/source-doc"

class processor_info:
    def __init__(self, lock, last_modified, instance):
        #Even if you're using multiple processes rather than threads use locking, just to be safe
        self.lock = lock
        self.last_modified = last_modified
        self.instance = {}

class processor_pool:
    """
    A hash table (LRU trimmed) that caches from XSLT transform tuples to prepared
    processor instances
    """
    def __init__(self):
        #Even if you're using multiple processes rather than threads use locking, just to be safe
        self._main_lock = thread.allocate_lock()
        self._granular_locks = {}
        self._processors = {}

    def get_processor(self, transform_hash, ext_functions=None, ext_elements=None):
        try:
            # First make sure we avoid race conditions...
            self._main_lock.acquire()
            if transform_hash not in self._granular_locks:
                self._granular_locks[transform_hash] = thread.allocate_lock()
        finally:
            self._main_lock.release()

        proc = None
        try:
            self._granular_locks[transform_hash].acquire()
            # If we do not have that stylesheet yet in the pool, let's add it
            if not self._processors.has_key(transform_hash):
                processor = Processor()
                self._processors[transform_hash] = processor_info(
                    thread.allocate_lock(), stat(transform_hash)[ST_MTIME], processor
                )
                for (ns, local), func in ext_functions.items():
                    processor.registerExtensionFunction(ns, local, func)
                for (ns, local), elem in ext_elements.items():
                    processor.registerExtensionElement(ns, local, elem)

                self._processors[transform_hash].instance.append_transform(iri.os_path_to_uri(transform_hash))
            # If the stylesheet has been modified, reload it
            elif stat(transform_hash)[ST_MTIME] != self._processors[transform_hash].last_modified:
                self._processors[transform_hash].instance.reset()
                self._processors[transform_hash].last_modified = stat(transform_hash)[ST_MTIME]
                self._processors[transform_hash].instance.append_transform(iri.os_path_to_uri(transform_hash))

            # now we can lock the processor...
            self._processors[transform_hash]['lock'].acquire()
            proc = self._processors[transform_hash].instance
        finally:
            self._granular_locks[transform_hash].release()
        # ... and return it
        return proc

    def release_processor(self, transform_hash):
        try:
            if self._processors[transform_hash]['lock'].locked():
               self._processors[transform_hash]['lock'].release()
        except: pass


class find_xslt_pis(sax.ContentHandler):
    def __init__(self, parser):
        parser.setContentHandler(self)
        self.parser = parser
        return

    def startDocument(self):
        self.ecount = 0
        self.xslt_pi = None

    def startElementNS(self, name, qname, attribs):
        self.ecount += 1
        if self.ecount == 2:
            #We're now within the doc proper, so we're done
            self.parser.setProperty(sax.PROPERTY_YIELD_RESULT, self.xslt_pi)
        return
            
    def processingInstruction(self, target, data):
        if target == u'xml-stylesheet':
            data = data.split()
            pseudo_attrs = {}
            for d in data:
                seg = d.split('=')
                if len(seg) == 2:
                    pseudo_attrs[seg[0]] = seg[1][1:-1]

            # PI must have both href, type pseudo-attributes;
            # type pseudo-attr must match valid XSLT types;
            # media pseudo-attr must match preferred media
            # (which can be None)
            if (pseudo_attrs.has_key('href')
                and pseudo_attrs.has_key('type')
                and pseudo_attrs['type'] in Processor.XSLT_IMT):
                    self.xslt_pi = pseudo_attrs['href']
                    self.parser.setProperty(sax.PROPERTY_YIELD_RESULT, self.xslt_pi)
        return


ACTIVE_FLAG = 'http://purl.org/xml3k/akara/transform/active'
SERVER_SIDE_FLAG = 'http://purl.org/xml3k/akara/transform/force-server-side'
CACHEABLE_FLAG = 'http://purl.org/xml3k/akara/transform/cacheable'

class xslt_transform_manager(object):
    """
    Middleware for XSLT transform processing.
    
    Also optionally checks for XSLT transform capability in the client and
    performs the transform on the server side if one is required, and the
    client can't do it
    """
    def __init__(self, app, use_wsgi_env=True, stock_xslt_params=None,
                 ext_modules=None):
        """
        use_wsgi_env - Optional bool determining whether to make the
            WSGI environment available to the XSLT as top level parameter
            overrides (e.g. wsgi:SCRIPT_NAME and wsgi:wsgi.url_scheme).
            Only passes on values it can handle (UTF-8 strings, Unicode,
            numbers, boolean, lists of "nodes").  Default to True
        stock_xslt_params - optional dict of dicts to also pass along as XSLT
            params.  The outer dict is onf the form: {<namespace>: <inner-dict>}
            And the inner dicts are of the form {pname: pvalue}.  The keys
            (pname) may be given as unicode objects if they have no namespace,
            or as (uri, localname) tuples if they do.  The values are
            (UTF-8 strings, Unicode, numbers, boolean, lists of "nodes").
            This is usually used for passing configuration info into XSLT
        ext_modules - Optional list of modules with XPath and XSLT extensions
        """
        #Set-up phase
        self.wrapped_app = app
        self.use_wsgi_env = use_wsgi_env
        self.stock_xslt_params = stock_xslt_params or {}
        self.ext_modules = ext_modules or []
        self.processor_cache = {}
        self.path_cache = {}
        return

    def __call__(self, environ, start_response):
        #Guess whether the client supports XML+XSLT?
        #See: http://copia.ogbuji.net/blog/2006-08-26/LazyWeb_Ho
        client_ua = environ.get('HTTP_USER_AGENT', '')
        path = environ['PATH_INFO']
        send_browser_xslt = True in [ ua_pat.match(client_ua) is not None
                                for ua_pat in USER_AGENT_REGEXEN ]

        #We'll hack a bit for dealing with Python's imperfect nested scopes.
        response_params = []
        def start_response_wrapper(status, response_headers, exc_info=None):
            #Assume response does not use XSLT; do not activate middleware
            environ[ACTIVE_FLAG] = False
            #Check for response content type to see whether it is XML
            for name, value in response_headers:
                #content-type value is a media type, defined as
                #media-type = type "/" subtype *( ";" parameter )
                media_type = value.split(';')[0]
                if ( name.lower() == 'content-type'
                     and MTYPE_PAT.match(media_type)):
                     #.endswith('/xml')
                     #      or media_type.find('/xml+') != -1 )):
                    environ[ACTIVE_FLAG] = True
            response_params.extend([status, response_headers, exc_info])
            #Replace any write() callable with a dummy that gives an error
            #The idea is to refuse support for apps that use write()
            def dummy_write(data):
                raise RuntimeError('applyxslt does not support the deprecated write() callable in WSGI apps')
            return dummy_write

        #Get the iterator from the application that will yield response
        #body fragments
        iterable = self.wrapped_app(environ, start_response_wrapper)
        (status, response_headers, exc_info) = response_params
        force_server_side = environ.get(SERVER_SIDE_FLAG, False)
        send_browser_xslt = send_browser_xslt and not force_server_side
        #import pprint; pprint.pprint(environ)

        #This function processes each chunk of output (simple string) from
        #the app, returning The modified chunk to be passed on to the server
        def next_response_block(response_iter):
            if send_browser_xslt or not environ[ACTIVE_FLAG]:
                #The client can handle XSLT, or it's not an XML source doc,
                #so nothing for this middleware to do
                start_response(status, response_headers, exc_info)
                for block in response_iter.next():
                    yield block
            elif path in self.path_cache:
                print >> sys.stderr, 'Using cached result for path', path
                imt, content = self.path_cache[path]
                response_headers.append(('content-type', imt))
                start_response(status, response_headers, exc_info)
                yield content
            else:
                yield produce_final_output(''.join(response_iter))
            return  

        #After the app has finished sending its response body fragments
        #if transform is required, it's necessary to send one more chunk,
        #with the fully transformed result
        def produce_final_output(response, response_headers=response_headers):
            log = sys.stderr
            if not send_browser_xslt and environ[ACTIVE_FLAG]:
                use_pi = False
                if force_server_side and force_server_side != True:
                    #True is a special flag meaning "don't delegate to the browser but still check for XSLT PIs"
                    xslt = force_server_side
                else:
                    #Check for a Stylesheet PI
                    parser = sax.reader()
                    parser.setFeature(sax.FEATURE_GENERATOR, True)
                    handler = find_xslt_pis(parser)
                    pi_iter = parser.parse(inputsource(response))
                    try:
                        #Note: only grabs the first PI.  Consider whether we should handle multiple
                        xslt = pi_iter.next()
                    except StopIteration:
                        xslt = None
                    use_pi = True
                if xslt:
                    xslt = xslt.encode('utf-8')
                    result = StringIO()
                    #self.xslt_sources = environ.get(
                    #    'wsgixml.applyxslt.xslt_sources', {})
                    source = InputSource.DefaultFactory.fromString(
                        response, uri=get_request_url(environ))
                    params = {}
                    for ns in self.stock_xslt_params:
                        params.update(setup_xslt_params(ns, self.stock_xslt_params[ns]))
                    start = time.time()

                    '''
                        processor = self.processorPool.get_processor(
                            stylesheet, self.ext_functions, self.ext_elements)
                        cherrypy.response.body = processor.run(
                            DefaultFactory.fromString(picket.document,
                                                      picket.uri),
                            topLevelParams=picket.parameters)
                        if self.default_content_type:
                            cherrypy.response.headers['Content-Type'] = self.default_content_type
                        if picket.content_type:
                            cherrypy.response.headers['Content-Type'] = picket.content_type
                    finally:
                        self.processorPool.release_processor(stylesheet)
                        '''


                    if xslt in self.processor_cache:
                        processor = self.processor_cache[xslt]
                        #Any transform would have already been loaded
                        use_pi = False
                        print >> log, 'Using cached processor instance for transform', xslt
                    else:
                        print >> log, 'Creating new processor instance for transform', xslt
                        processor = Processor.Processor()
                        if self.ext_modules:
                            processor.registerExtensionModules(self.ext_modules)
                        if self.use_wsgi_env:
                            params.update(setup_xslt_params(WSGI_NS, environ))
                        #srcAsUri = OsPathToUri()
                        #if False:
                        if environ.has_key('paste.recursive.include'):
                            #paste's recursive facilities are available, to
                            #so we can get the XSLT with a middleware call
                            #rather than a full Web invocation
                            #print environ['paste.recursive.include']
                            xslt_resp = environ['paste.recursive.include'](xslt)
                            #FIXME: this should be relative to the XSLT, not XML
                            #print xslt_resp, xslt_resp.body
                            isrc = InputSource.DefaultFactory.fromString(
                                xslt_resp.body, get_request_url(environ))
                            processor.appendStylesheet(isrc)
                        else:
                            #We have to make a full Web call to get the XSLT.
                            #4Suite will do that for us in processing the PI
                            if not use_pi:
                                uri = Uri.Absolutize(xslt, get_request_url(environ))
                                isrc = InputSource.DefaultFactory.fromUri(uri)
                                processor.appendStylesheet(isrc)
                        self.processor_cache[xslt] = processor
                    processor.run(source, outputStream=result,
                                  ignorePis=not use_pi, topLevelParams=params)

                    #Strip content-length if present (needs to be
                    #recalculated by server)
                    #Also strip content-type, which will be replaced below
                    response_headers = [ (name, value)
                        for name, value in response_headers
                            if ( name.lower()
                                 not in ['content-length', 'content-type'])
                    ]
                    #Put in the updated content type
                    imt = processor.outputParams.mediaType
                    content = result.getvalue()
                    if environ.get(CACHEABLE_FLAG):
                        self.path_cache[path] = imt, content
                    response_headers.append(('content-type', imt))
                    start_response(status, response_headers, exc_info)
                    end = time.time()
                    print >> log, '%s: elapsed time: %0.3f\n'%(xslt, end-start)
                    #environ['wsgi.errors'].write('%s: elapsed time: %0.3f\n'%(xslt, end-start))
                    return content
                    
            #If it reaches this point, no XSLT was applied.
            return

        return iterwrapper(iterable, next_response_block)





class PicketFilter(BaseFilter):
    """
    A filter that applies XSLT templates to XML content
    For any published method with this filter attached, return an
    instance of the Picket class if you want an XSLT transform invoked.
    The string output of the transform becomes the Web response body
    """
    def __init__(self, default_stylesheet=None, default_content_type=None, extension_dir=None):
        self.processorPool = ProcessorPool()
        self.default_stylesheet = default_stylesheet
        self.default_content_type = default_content_type
        self.extensionDir = extension_dir #cherrypy.config.get("picket.extensionDir")
        self.ext_functions, self.ext_elements = _getExtensions(self.extensionDir)

    def before_finalize(self):
        picket = cherrypy.response.body

        if not isinstance(picket, Picket): return

        stylesheet = self.default_stylesheet

        if picket.stylesheet:
            stylesheet = picket.stylesheet

        if stylesheet is None:
            # If a stylesheet was not set, then raise an error
            raise ValueError, "Missing XSLT stylesheet"

        extDir = cherrypy.config.get("picket.extensionDir")
        if extDir != self.extensionDir:
            self.extensionDir =extDir
            self.ext_functions, self.ext_elements = _getExtensions(self.extensionDir)

        try:
            processor = self.processorPool.get_processor(
                stylesheet, self.ext_functions, self.ext_elements)
            cherrypy.response.body = processor.run(
                DefaultFactory.fromString(picket.document,
                                          picket.uri),
                topLevelParams=picket.parameters)
            if self.default_content_type:
                cherrypy.response.headers['Content-Type'] = self.default_content_type
            if picket.content_type:
                cherrypy.response.headers['Content-Type'] = picket.content_type
        finally:
            self.processorPool.release_processor(stylesheet)


def _getExtensions(extensionDirectory):
    import glob
    import os

    retval = ({}, {})
    try:
        os.path.isdir(extensionDirectory)
    except TypeError:
        return retval

    for extensionPath in glob.glob(extensionDirectory + "/*.py"):
        try:
            ns = {}
            execfile(extensionPath, ns, ns)
            retval[0].update(ns.get("ExtFunctions", {}))
            retval[1].update(ns.get("ExtElements", {}))
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            pass

    return retval








import os

from wsgixml.applyxslt import xsltize
import pkg_resources

from cStringIO import StringIO

from Ft.Lib import Uri, UriException
from Ft.Xml import InputSource, CreateInputSource
from Ft.Xml.InputSource import InputSourceFactory
from Ft.Xml.Xslt.Processor import Processor
from Ft.Xml.Xslt.StylesheetReader import StylesheetReader
from Ft.Xml import Domlette, Parse

from pprint import pprint

class LocalTemplateResolver(Uri.FtUriResolver):

    def normalize(self, uri_ref, base_uri):
        return Uri.Absolutize(uri_ref, base_uri)

    def _orig_resolve(self, uri, baseUri=None):
        """
        This function takes a URI or a URI reference plus a base URI, produces
        a normalized URI using the normalize function if a base URI was given,
        then attempts to obtain access to an entity representing the resource
        identified by the resulting URI, returning the entity as a stream (a
        Python file-like object).

        Raises a UriException if the URI scheme is unsupported or if a stream
        could not be obtained for any reason.
        """
        if baseUri is not None:
            uri = self.normalize(uri, baseUri)
            scheme = Uri.GetScheme(uri)
        else:
            scheme = Uri.GetScheme(uri)
            # since we didn't use normalize(), we need to verify here
            if scheme not in Uri.DEFAULT_URI_SCHEMES:
                if scheme is None:
                    raise ValueError('When the URI to resolve is a relative '
                        'reference, it must be accompanied by a base URI.')
                else:
                    raise UriException(UriException.UNSUPPORTED_SCHEME,
                                       scheme=scheme,
                                       resolver=self.__class__.__name__)

        # Bypass urllib for opening local files. This means we don't get all
        # the extra metadata that urllib adds to the stream (Last-modified,
        # Content-Length, a poorly guessed Content-Type, and the URI), but
        # we also avoid its potentially time-consuming socket.gethostbyname()
        # calls, which aren't even warranted and are part of urllib's dubious
        # interpretation of RFC 1738.
        if scheme == 'file':
            path = Uri.UriToOsPath(uri, attemptAbsolute=False)
            try:
                stream = file(path, 'rb')
            except IOError, e:
                raise UriException(UriException.RESOURCE_ERROR,
                                   loc='%s (%s)' % (uri, path),
                                   uri=uri, msg=str(e))
        else:
            # urllib2.urlopen, wrapped by us, will suffice for http, ftp,
            # data and gopher
            try:
                stream = Uri.UrlOpen(uri)
            except IOError, e:
                raise UriException(UriException.RESOURCE_ERROR,
                                   uri=uri, loc=uri, msg=str(e))
        return stream

    def resolve(self, uri, base_uri=None):
        here = os.path.abspath('.')
        if uri.startswith('local:'):
            uri = uri[6:]
            resource = os.path.join(self.templates, uri)
            if os.path.exists(resource):
                return file(resource, 'rb')
            raise UriException(UriException.RESOURCE_ERROR,
                               uri=uri, loc=uri,
                               msg="The file did not exist in '%s'" % self.templates)
        elif uri.startswith('pkg:'):
            # format: package#path/to/file.xslt
            usage = 'usage: package_name#path/to/file'
            uri = uri[4:]
            package, sep, path = uri.partition('#')
            if not package or path:
                raise UriException(
                    UriException.RESOURCE_ERROR,
                    uri=uri, loc=uri,
                    msg="Invalid pkg_resources uri. \n %s" % usage
                )
            if pkg_resources.resource_exists(package, path):
                return pkg_resources.resource_stream(package, path)
            raise UriException(
                UriException.RESOURCE_ERROR,
                uri=uri, loc=uri,
                msg="'%s' was not found in the python package '%s'" % (path, package)
            )
        else:
            return self._orig_resolve(uri, base_uri)


XParams = 'xsltemplate.params'
XTemplate = 'xsltemplate.template'
XSource = 'xsltemplate.source'

class TemplateMiddleware(object):

    def __init__(self, app_conf, app, **kw):

        self.ns = unicode(app_conf.get('xsltemplate_namespace',
                                       'http://ionrock.org/ns/xsltemplate'))
        if app_conf.get('use_index_xml'):
            self.content = app_conf['use_index_xml']
        self.template_key = XTemplate
        self.params_key = XParams
        self.source_key = XSource
        self.tdir = app_conf.get('template_directory', 'templates')
        self.resolver = LocalTemplateResolver()
        self.resolver.templates = self.tdir
        self.xslt_factory = InputSourceFactory(resolver=self.resolver)
        self.rs = '%s.xslt'
        self.app = app
        if kw.get('extensions'):
            self.extensions = kw['extensions']
        else:
            self.extensions = None

    def start_response(self, status, headers, exc_info=None):
        self.status = status
        self.headers = headers
        self.exc_info = exc_info

    def __call__(self, environ, start_response):
        source = ''.join(self.app(environ, self.start_response))
        if not source and self.content:
            source = self.content
        if environ.get(self.template_key):
            xslt = environ[self.template_key]
            params = environ.get(self.params_key, {})
            source = self.do_render(source, xslt, params)
        for i, value in enumerate(self.headers):
            k, v = value
            if k.lower() == 'content-length':
                del self.headers[i]
        start_response(self.status, self.headers, self.exc_info)
        return [source]


    def get_processor(self):
        proc = Processor()
        if self.extensions:
            for ext in self.extensions:
                proc.registerExtensionFunction(*(ext))
        return proc

    def get(self, fn):
        if fn.startswith('pkg://'):
            package, sep, path = fn[6:].partition('#')
            if pkg_resources.resource_exists(package, path):
                return self.xslt_factory.fromString(pkg_resources.resource_string(package, path))
        path = Uri.OsPathToUri(os.path.join(self.tdir, fn))
        try:
            xslt = self.xslt_factory.fromUri(path)
        except UriException, e:
            xslt = self.xslt_factory.fromString(
                fn, Uri.OsPathToUri(os.path.abspath('.'))
            )
        return xslt

    def run(self, xml, xslt, params):
        proc = self.get_processor()
        xml = CreateInputSource(xml)
        xslt = self.get(xslt)
        proc.appendStylesheet(xslt)
        out = proc.run(xml, topLevelParams=params)
        del proc
        return out

    def do_render(self, xml, xslt, params):
        params['check_params'] = "Yup they are working!"
        nodes = {}
        for k, v in params.items():
            if isinstance(v, list):
                nodes[k] = v
        params = self.setup_xslt_params(params)
        for k, v in nodes.items():
            params[(self.ns, k)] = v
        return self.run(xml, xslt, params=params)

    def setup_xslt_params(self, params):
        xsltparams = dict([ ((self.ns, k), params[k])
                            for k in params
                            if xsltize(params[k]) is not None ])
        return xsltparams

class IndexXMLMiddleware(object):
    def __init__(self, app_conf, app):
        self.app_conf = app_conf
        self.content = '<?xml version="1.0"?><page />'
        if self.app_conf.get('index_xml'):
            if os.path.exists(self.app_conf['index_xml']):
                self.content = open(self.app_conf['index_xml'],'rb').read()
        self.app = app

    def start_response(self, status, headers, exc_info=None):
        self.status = status
        self.headers = headers
        self.exc_info = exc_info

    def __call__(self, environ, start_response, exc_info=None):
        c = self.app(environ, self.start_response)
        start_response(self.status, self.headers, self.exc_info)
        if c:
            return c
        return [self.content]

def set_params(environ, params):
    values = environ.get(XParams, {})
    values.update(params)
    environ[XParams] = values

def set_template(environ, template):
    environ[XTemplate] = template

def node_set(xml):
    return Parse(xml)

class TemplateConstants(object):

    def __init__(self, constants_dict, app, use_environ=True):
        self.constants = constants_dict
        self.use_environ = use_environ
        self.app = app

    def __call__(self, environ, start_response):
        if self.use_environ:
            # strip out xsltemplates params
            set_params(environ,
                       dict([ (k, v) for k, v in environ.items()
                              if not k.startswith(XParams) ]))
        set_params(environ, self.constants)
        return self.app(environ, start_response)



# -*- coding: iso-8859-1 -*-
# 
"""
moin2atomentries.py (Akara demo)

Accesses a Moin wiki (via akara.restwrap.moin) to use as a source for a Web feed

A RESTful wrapper for MoinMoin wikis

Copyright 2009 Uche Ogbuji
This file is part of the open source Akara project,
provided under the Apache 2.0 license.
See the files LICENSE and NOTICE for details.
Project home, documentation, distributions: http://wiki.xml3k.org/Akara

@copyright: 2009 by Uche ogbuji <uche@ogbuji.net>

Can be launched from the command line, e.g.:
    python demo/moin2atomentries.py http://restwrap.mywiki.example.com/ /path/to/output/dir
"""
#
#Detailed license and copyright information: http://4suite.org/COPYRIGHT

from __future__ import with_statement

import os
import re
import pprint
import httplib
import urllib, urllib2
import datetime
from wsgiref.util import shift_path_info, request_uri
from string import Template
from cStringIO import StringIO
from functools import partial
from itertools import *
from contextlib import closing

from dateutil.parser import parse as dateparse
import pytz

import amara
from amara import bindery
from amara.namespaces import *
from amara.writers.struct import *
from amara.bindery.html import parse as htmlparse
from amara.lib.iri import split_fragment
from amara.bindery.util import dispatcher, node_handler, property_sequence_getter

WIKITEXT_IMT = 'text/plain'
DOCBOOK_IMT = 'application/docbook+xml'
RDF_IMT = 'application/rdf+xml'
DEFAULT_TZ = pytz.timezone('UTC')

import sys
import SocketServer
from wsgiref import simple_server

#aname = partial(property_sequence_getter, u"name")
#aemail = partial(property_sequence_getter, u"email")
#auri = partial(property_sequence_getter, u"uri")

class author(object):
    def __init__(self, para):
        self.name = unicode(para.ulink)
        self.email = unicode(para.ulink[1])
        self.uri = para.ulink.url
        return

UNSUPPORTED_IN_FILENAME = re.compile('\W')
LINK_PATTERN = u'http://zepheira.com/publications/news/#%s'

def pathsegment(relative):
    return UNSUPPORTED_IN_FILENAME.sub('_', relative)

def handle_page(uri, page, outputdir, relative, output):
    #tags = [u"xml", u"python", u"atom"]
    print >> sys.stderr, 'Processing ', uri
    title = unicode(page.article.section[0].title)
    sections = dict([ (unicode(s.title), s) for s in page.article.section ])
    #print sections
    summary = sections["entry:summary"]
    content = sections["entry:content"]
    tags = [ g for g in page.article.section.glosslist.glossentry if unicode(g.glossterm) == u'entry:tags' ]
    if tags: tags = [ gd.para.xml_select(u'string(.)').strip() for gd in tags[0].glossdef ]
    authors = [ a
        for a in page.article.section.glosslist.glossentry
        if unicode(a.glossterm) == u'entry:authors'
    ]
    if authors: authors = [ author(gd.para) for gd in authors[0].glossdef ]
    #title = article.xml_select(u'section[@title = ]')

    revdate = dateparse(unicode(page.article.articleinfo.revhistory.revision.date))
    if revdate.tzinfo == None: revdate = revdate.replace(tzinfo=DEFAULT_TZ)

    eid = LINK_PATTERN%unicode(uri.rsplit(u'/')[-1])
    w = structwriter(indent=u"yes", stream=output)
    w.feed(
    ROOT(
        E((ATOM_NAMESPACE, u'entry'), {(XML_NAMESPACE, u'xml:lang'): u'en'},
            #E(u'link', {u'href': u'/blog'}),
            E(u'link', {u'href': unicode(uri), u'rel': u'edit'}),
            E(u'link', {u'href': eid, u'rel': u'alternate', u'title': u"Permalink"}),
            E(u'id', eid),
            E(u'title', title),
            #FIXME: Use updated time from feed
            E(u'updated', unicode(revdate.isoformat())),
            #E(u'updated', datetime.datetime.now().isoformat()),
            #E(u'updated', page.updated),
            ( E(u'category', {u'term': t}) for t in tags ),
            ( E(u'author',
                E(u'name', a.name),
                E(u'uri', a.uri),
                E(u'email', a.email),
            ) for a in authors ),
            E(u'summary', {u'type': u'xhtml'},
                E((XHTML_NAMESPACE, u'div'),
                    CONTENT.dispatch(summary)
                )
            ),
            E(u'content', {u'type': u'xhtml'},
                E((XHTML_NAMESPACE, u'div'),
                    CONTENT.dispatch(content)
                )
            ),
        ),
    ))
    return


class content_handlers(dispatcher):
    @node_handler(u'section/title')
    def section_title(self, node):
        #Ignore this node
        raise StopIteration

    @node_handler(u'para')
    def para(self, node):
        #print 'content_handlers.para'
        yield E((XHTML_NAMESPACE, u'p'),
            chain(*imap(self.dispatch, node.xml_children))
        )

    @node_handler(u'code')
    def code(self, node):
        yield E((XHTML_NAMESPACE, u'code'),
            chain(*imap(self.dispatch, node.xml_children))
        )

    @node_handler(u'emphasis')
    def emphasis(self, node):
        #print dict(node.xml_attributes)
        ename = u'strong' if node.xml_attributes.get((None, u'role')) == u'strong' else u'em'
        yield E((XHTML_NAMESPACE, ename),
            chain(*imap(self.dispatch, node.xml_children))
        )

    @node_handler(u'screen')
    def screen(self, node):
        yield E((XHTML_NAMESPACE, u'pre'),
            chain(*imap(self.dispatch, node.xml_children))
        )

    @node_handler(u'ulink')
    def a(self, node):
        yield E((XHTML_NAMESPACE, u'a'), {u'href': node.url},
            chain(*imap(self.dispatch, node.xml_children))
        )

    #@node_handler(u'*', priority=-1)
    #def etc(self, node):

CONTENT = content_handlers()
OUTPUTPATTERN = 'MOIN.%s.atom'

def moin2atomentries(wikibase, outputdir, rewrite, pattern):
    wikibase_len = len(rewrite)
    if pattern: pattern = re.compile(pattern)
    #print (wikibase, outputdir, rewrite)
    req = urllib2.Request(wikibase, headers={'Accept': RDF_IMT})
    with closing(urllib2.urlopen(req)) as resp:
        feed = bindery.parse(resp)
    for item in feed.RDF.channel.items.Seq.li:
        uri = split_fragment(item.resource)[0]
        relative = uri[wikibase_len:]
        print >> sys.stderr, uri, relative
        if pattern and not pattern.match(relative):
            continue
        if rewrite:
            uri = uri.replace(rewrite, wikibase)
        req = urllib2.Request(uri, headers={'Accept': DOCBOOK_IMT})
        with closing(urllib2.urlopen(req)) as resp:
            page = bindery.parse(resp)
        entrydate = dateparse(unicode(page.article.articleinfo.revhistory.revision.date))
        if entrydate.tzinfo == None: entrydate = entrydate.replace(tzinfo=DEFAULT_TZ)
        output = os.path.join(outputdir, OUTPUTPATTERN%pathsegment(relative))
        if os.access(output, os.R_OK):
            lastrev = dateparse(unicode(bindery.parse(output).entry.updated))
            if lastrev.tzinfo == None: lastrev = lastrev.replace(tzinfo=DEFAULT_TZ)
            if (entrydate == lastrev):
                print >> sys.stderr, 'Not updated.  Skipped...'
                continue
        print >> sys.stderr, 'Writing to ', output
        with open(output, 'w') as output:
            handle_page(uri, page, outputdir, relative, output)
    return

#Ideas borrowed from
# http://www.artima.com/forums/flat.jsp?forum=106&thread=4829

#FIXME: A lot of this is copied boilerplate that neds to be cleaned up

def command_line_prep():
    from optparse import OptionParser
    usage = "%prog [options] wikibase outputdir"
    parser = OptionParser(usage=usage)
    parser.add_option("-p", "--pattern",
                      action="store", type="string", dest="pattern",
                      help="limit the pages treated as Atom entries to those matching this pattern")
    return parser


def main(argv=None):
    #But with better integration of entry points
    if argv is None:
        argv = sys.argv
    # By default, optparse usage errors are terminated by SystemExit
    try:
        optparser = command_line_prep()
        options, args = optparser.parse_args(argv[1:])
        # Process mandatory arguments with IndexError try...except blocks
        try:
            wikibase = args[0]
            try:
                outputdir = args[1]
            except IndexError:
                optparser.error("Missing output directory")
        except IndexError:
            optparser.error("Missing Wiki base URL")
    except SystemExit, status:
        return status
    rewrite = args[2] if len(args) > 1 else None

    # Perform additional setup work here before dispatching to run()
    # Detectable errors encountered here should be handled and a status
    # code of 1 should be returned. Note, this would be the default code
    # for a SystemExit exception with a string message.
    pattern = options.pattern and options.pattern.decode('utf-8')

    moin2atomentries(wikibase, outputdir, rewrite, pattern)
    return


if __name__ == "__main__":
    sys.exit(main(sys.argv))

